@@ -34,7 +34,6 @@ Available unstable (nightly-only) flags:
     -Z offline          -- Offline mode that does not perform network requests
     -Z unstable-options -- Allow the usage of unstable options such as --registry
     -Z config-profile   -- Read profiles from .cargo/config files
-    -Z compile-progress -- Display a progress bar while compiling
 
 Run with 'cargo -Z [FLAG] [SUBCOMMAND]'"
         );
@@ -43,20 +42,8 @@ Run with 'cargo -Z [FLAG] [SUBCOMMAND]'"
 
     let is_verbose = args.occurrences_of("verbose") > 0;
     if args.is_present("version") {
-        let version = cargo::version();
-        println!("{}", version);
-        if is_verbose {
-            println!(
-                "release: {}.{}.{}",
-                version.major, version.minor, version.patch
-            );
-            if let Some(ref cfg) = version.cfg_info {
-                if let Some(ref ci) = cfg.commit_info {
-                    println!("commit-hash: {}", ci.commit_hash);
-                    println!("commit-date: {}", ci.commit_date);
-                }
-            }
-        }
+        let version = get_version_string(is_verbose);
+        print!("{}", version);
         return Ok(());
     }
 
@@ -92,6 +79,25 @@ Run with 'cargo -Z [FLAG] [SUBCOMMAND]'"
     execute_subcommand(config, &args)
 }
 
+pub fn get_version_string(is_verbose: bool) -> String {
+    let version = cargo::version();
+    let mut version_string = String::from(version.to_string());
+    version_string.push_str("\n");
+    if is_verbose {
+        version_string.push_str(&format!(
+            "release: {}.{}.{}\n",
+            version.major, version.minor, version.patch
+        ));
+        if let Some(ref cfg) = version.cfg_info {
+            if let Some(ref ci) = cfg.commit_info {
+                version_string.push_str(&format!("commit-hash: {}\n", ci.commit_hash));
+                version_string.push_str(&format!("commit-date: {}\n", ci.commit_date));
+            }
+        }
+    }
+    version_string
+}
+
 fn expand_aliases(
     config: &mut Config,
     args: ArgMatches<'static>,
@@ -179,18 +185,18 @@ OPTIONS:
 {unified}
 
 Some common cargo commands are (see all commands with --list):
-    build       Compile the current project
-    check       Analyze the current project and report errors, but don't build object files
+    build       Compile the current package
+    check       Analyze the current package and report errors, but don't build object files
     clean       Remove the target directory
-    doc         Build this project's and its dependencies' documentation
-    new         Create a new cargo project
-    init        Create a new cargo project in an existing directory
+    doc         Build this package's and its dependencies' documentation
+    new         Create a new cargo package
+    init        Create a new cargo package in an existing directory
     run         Build and execute src/main.rs
     test        Run the tests
     bench       Run the benchmarks
     update      Update dependencies listed in Cargo.lock
     search      Search registry for crates
-    publish     Package and upload this project to the registry
+    publish     Package and upload this package to the registry
     install     Install a Rust binary
     uninstall   Uninstall a Rust binary
 
