@@ -5,8 +5,11 @@ use cargo::print_json;
 pub fn cli() -> App {
     subcommand("read-manifest")
         .about(
-            "Deprecated, use `cargo metadata --no-deps` instead.
-Print a JSON representation of a Cargo.toml manifest.",
+            "\
+Print a JSON representation of a Cargo.toml manifest.
+
+Deprecated, use `cargo metadata --no-deps` instead.\
+",
         )
         .arg_manifest_path()
 }
