@@ -15,7 +15,7 @@ use core::{Dependency, PackageId, PackageIdSpec, SourceId, Summary};
 use core::{Edition, Feature, Features, WorkspaceConfig};
 use util::errors::*;
 use util::toml::TomlManifest;
-use util::Config;
+use util::{Config, Filesystem, short_hash};
 
 pub enum EitherManifest {
     Real(Manifest),
@@ -44,6 +44,7 @@ pub struct Manifest {
     edition: Edition,
     im_a_teapot: Option<bool>,
     default_run: Option<String>,
+    metabuild: Option<Vec<String>>,
 }
 
 /// When parsing `Cargo.toml`, some warnings should silenced
@@ -190,7 +191,7 @@ pub struct Target {
     // as it's absolute currently and is otherwise a little too brittle for
     // causing rebuilds. Instead the hash for the path that we send to the
     // compiler is handled elsewhere.
-    src_path: NonHashedPathBuf,
+    src_path: TargetSourcePath,
     required_features: Option<Vec<String>>,
     tested: bool,
     benched: bool,
@@ -202,19 +203,50 @@ pub struct Target {
 }
 
 #[derive(Clone, PartialEq, Eq)]
-struct NonHashedPathBuf {
-    path: PathBuf,
+pub enum TargetSourcePath {
+    Path(PathBuf),
+    Metabuild,
 }
 
-impl Hash for NonHashedPathBuf {
+impl TargetSourcePath {
+    pub fn path(&self) -> &Path {
+        match self {
+            TargetSourcePath::Path(path) => path.as_ref(),
+            TargetSourcePath::Metabuild => panic!("metabuild not expected"),
+        }
+    }
+
+    pub fn is_path(&self) -> bool {
+        match self {
+            TargetSourcePath::Path(_) => true,
+            _ => false,
+        }
+    }
+}
+
+impl Hash for TargetSourcePath {
     fn hash<H: Hasher>(&self, _: &mut H) {
         // ...
     }
 }
 
-impl fmt::Debug for NonHashedPathBuf {
+impl fmt::Debug for TargetSourcePath {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        self.path.fmt(f)
+        match self {
+            TargetSourcePath::Path(path) => path.fmt(f),
+            TargetSourcePath::Metabuild => "metabuild".fmt(f),
+        }
+    }
+}
+
+impl From<PathBuf> for TargetSourcePath {
+    fn from(path: PathBuf) -> Self {
+        assert!(
+            path.is_absolute(),
+            "`{}` is not absolute",
+            path.display()
+        );
+        TargetSourcePath::Path(path)
     }
 }
 
@@ -229,6 +261,8 @@ struct SerializedTarget<'a> {
     name: &'a str,
     src_path: &'a PathBuf,
     edition: &'a str,
+    #[serde(rename = "required-features", skip_serializing_if = "Option::is_none")]
+    required_features: Option<Vec<&'a str>>,
 }
 
 impl ser::Serialize for Target {
@@ -237,8 +271,12 @@ impl ser::Serialize for Target {
             kind: &self.kind,
             crate_types: self.rustc_crate_types(),
             name: &self.name,
-            src_path: &self.src_path.path,
-            edition: &self.edition.to_string()
+            src_path: &self.src_path.path().to_path_buf(),
+            edition: &self.edition.to_string(),
+            required_features: self
+                .required_features
+                .as_ref()
+                .map(|rf| rf.iter().map(|s| &**s).collect()),
         }.serialize(s)
     }
 }
@@ -247,34 +285,43 @@ compact_debug! {
     impl fmt::Debug for Target {
         fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
             let (default, default_name) = {
-                let src = self.src_path().to_path_buf();
                 match &self.kind {
                     TargetKind::Lib(kinds) => {
                         (
                             Target::lib_target(
                                 &self.name,
                                 kinds.clone(),
-                                src.clone(),
-                                Edition::Edition2015,
+                                self.src_path().path().to_path_buf(),
+                                self.edition,
                             ),
-                            format!("lib_target({:?}, {:?}, {:?})",
-                                    self.name, kinds, src),
+                            format!("lib_target({:?}, {:?}, {:?}, {:?})",
+                                    self.name, kinds, self.src_path, self.edition),
                         )
                     }
                     TargetKind::CustomBuild => {
-                        (
-                            Target::custom_build_target(
-                                &self.name,
-                                src.clone(),
-                                Edition::Edition2015,
-                            ),
-                            format!("custom_build_target({:?}, {:?})",
-                                    self.name, src),
-                        )
+                        match self.src_path {
+                            TargetSourcePath::Path(ref path) => {
+                                (
+                                    Target::custom_build_target(
+                                        &self.name,
+                                        path.to_path_buf(),
+                                        self.edition,
+                                    ),
+                                    format!("custom_build_target({:?}, {:?}, {:?})",
+                                            self.name, path, self.edition),
+                                )
+                            }
+                            TargetSourcePath::Metabuild => {
+                                (
+                                    Target::metabuild_target(&self.name),
+                                    format!("metabuild_target({:?})", self.name),
+                                )
+                            }
+                        }
                     }
                     _ => (
-                        Target::with_path(src.clone(), Edition::Edition2015),
-                        format!("with_path({:?})", src),
+                        Target::new(self.src_path.clone(), self.edition),
+                        format!("with_path({:?}, {:?})", self.src_path, self.edition),
                     ),
                 }
             };
@@ -315,6 +362,7 @@ impl Manifest {
         im_a_teapot: Option<bool>,
         default_run: Option<String>,
         original: Rc<TomlManifest>,
+        metabuild: Option<Vec<String>>,
     ) -> Manifest {
         Manifest {
             summary,
@@ -336,6 +384,7 @@ impl Manifest {
             im_a_teapot,
             default_run,
             publish_lockfile,
+            metabuild,
         }
     }
 
@@ -458,6 +507,18 @@ impl Manifest {
     pub fn default_run(&self) -> Option<&str> {
         self.default_run.as_ref().map(|s| &s[..])
     }
+
+    pub fn metabuild(&self) -> Option<&Vec<String>> {
+        self.metabuild.as_ref()
+    }
+
+    pub fn metabuild_path(&self, target_dir: Filesystem) -> PathBuf {
+        let hash = short_hash(self.package_id());
+        target_dir
+            .into_path_unlocked()
+            .join(".metabuild")
+            .join(format!("metabuild-{}-{}.rs", self.name(), hash))
+    }
 }
 
 impl VirtualManifest {
@@ -502,16 +563,11 @@ impl VirtualManifest {
 }
 
 impl Target {
-    fn with_path(src_path: PathBuf, edition: Edition) -> Target {
-        assert!(
-            src_path.is_absolute(),
-            "`{}` is not absolute",
-            src_path.display()
-        );
+    fn new(src_path: TargetSourcePath, edition: Edition) -> Target {
         Target {
             kind: TargetKind::Bin,
             name: String::new(),
-            src_path: NonHashedPathBuf { path: src_path },
+            src_path,
             required_features: None,
             doc: false,
             doctest: false,
@@ -523,6 +579,10 @@ impl Target {
         }
     }
 
+    fn with_path(src_path: PathBuf, edition: Edition) -> Target {
+        Target::new(TargetSourcePath::from(src_path), edition)
+    }
+
     pub fn lib_target(
         name: &str,
         crate_targets: Vec<LibKind>,
@@ -569,6 +629,17 @@ impl Target {
         }
     }
 
+    pub fn metabuild_target(name: &str) -> Target {
+        Target {
+            kind: TargetKind::CustomBuild,
+            name: name.to_string(),
+            for_host: true,
+            benched: false,
+            tested: false,
+            ..Target::new(TargetSourcePath::Metabuild, Edition::Edition2015)
+        }
+    }
+
     pub fn example_target(
         name: &str,
         crate_targets: Vec<LibKind>,
@@ -628,8 +699,11 @@ impl Target {
     pub fn crate_name(&self) -> String {
         self.name.replace("-", "_")
     }
-    pub fn src_path(&self) -> &Path {
-        &self.src_path.path
+    pub fn src_path(&self) -> &TargetSourcePath {
+        &self.src_path
+    }
+    pub fn set_src_path(&mut self, src_path: TargetSourcePath) {
+        self.src_path = src_path;
     }
     pub fn required_features(&self) -> Option<&Vec<String>> {
         self.required_features.as_ref()
