@@ -3,34 +3,49 @@ use std::fs;
 use std::io;
 use std::path::{Path, PathBuf};
 
-use core::{Package, SourceId, PackageId, EitherManifest};
+use core::{EitherManifest, Package, PackageId, SourceId};
 use util::{self, Config};
-use util::errors::{CargoResult, CargoError};
+use util::errors::{CargoError, CargoResult};
 use util::important_paths::find_project_manifest_exact;
 use util::toml::read_manifest;
 
-pub fn read_package(path: &Path, source_id: &SourceId, config: &Config)
-                    -> CargoResult<(Package, Vec<PathBuf>)> {
-    trace!("read_package; path={}; source-id={}", path.display(), source_id);
+pub fn read_package(
+    path: &Path,
+    source_id: &SourceId,
+    config: &Config,
+) -> CargoResult<(Package, Vec<PathBuf>)> {
+    trace!(
+        "read_package; path={}; source-id={}",
+        path.display(),
+        source_id
+    );
     let (manifest, nested) = read_manifest(path, source_id, config)?;
     let manifest = match manifest {
         EitherManifest::Real(manifest) => manifest,
-        EitherManifest::Virtual(..) => {
-            bail!("found a virtual manifest at `{}` instead of a package \
-                   manifest", path.display())
-        }
+        EitherManifest::Virtual(..) => bail!(
+            "found a virtual manifest at `{}` instead of a package \
+             manifest",
+            path.display()
+        ),
     };
 
     Ok((Package::new(manifest, path), nested))
 }
 
-pub fn read_packages(path: &Path, source_id: &SourceId, config: &Config)
-                     -> CargoResult<Vec<Package>> {
+pub fn read_packages(
+    path: &Path,
+    source_id: &SourceId,
+    config: &Config,
+) -> CargoResult<Vec<Package>> {
     let mut all_packages = HashMap::new();
     let mut visited = HashSet::<PathBuf>::new();
     let mut errors = Vec::<CargoError>::new();
 
-    trace!("looking for root package: {}, source_id={}", path.display(), source_id);
+    trace!(
+        "looking for root package: {}, source_id={}",
+        path.display(),
+        source_id
+    );
 
     walk(path, &mut |dir| {
         trace!("looking for child package: {}", dir.display());
@@ -39,24 +54,31 @@ pub fn read_packages(path: &Path, source_id: &SourceId, config: &Config)
         if dir != path {
             let name = dir.file_name().and_then(|s| s.to_str());
             if name.map(|s| s.starts_with('.')) == Some(true) {
-                return Ok(false)
+                return Ok(false);
             }
 
             // Don't automatically discover packages across git submodules
             if fs::metadata(&dir.join(".git")).is_ok() {
-                return Ok(false)
+                return Ok(false);
             }
         }
 
         // Don't ever look at target directories
-        if dir.file_name().and_then(|s| s.to_str()) == Some("target") &&
-           has_manifest(dir.parent().unwrap()) {
-            return Ok(false)
+        if dir.file_name().and_then(|s| s.to_str()) == Some("target")
+            && has_manifest(dir.parent().unwrap())
+        {
+            return Ok(false);
         }
 
         if has_manifest(dir) {
-            read_nested_packages(dir, &mut all_packages, source_id, config,
-                                      &mut visited, &mut errors)?;
+            read_nested_packages(
+                dir,
+                &mut all_packages,
+                source_id,
+                config,
+                &mut visited,
+                &mut errors,
+            )?;
         }
         Ok(true)
     })?;
@@ -64,31 +86,31 @@ pub fn read_packages(path: &Path, source_id: &SourceId, config: &Config)
     if all_packages.is_empty() {
         match errors.pop() {
             Some(err) => Err(err),
-            None => Err(format_err!("Could not find Cargo.toml in `{}`", path.display())),
+            None => Err(format_err!(
+                "Could not find Cargo.toml in `{}`",
+                path.display()
+            )),
         }
     } else {
         Ok(all_packages.into_iter().map(|(_, v)| v).collect())
     }
 }
 
-fn walk(path: &Path, callback: &mut FnMut(&Path) -> CargoResult<bool>)
-        -> CargoResult<()> {
+fn walk(path: &Path, callback: &mut FnMut(&Path) -> CargoResult<bool>) -> CargoResult<()> {
     if !callback(path)? {
         trace!("not processing {}", path.display());
-        return Ok(())
+        return Ok(());
     }
 
     // Ignore any permission denied errors because temporary directories
     // can often have some weird permissions on them.
     let dirs = match fs::read_dir(path) {
         Ok(dirs) => dirs,
-        Err(ref e) if e.kind() == io::ErrorKind::PermissionDenied => {
-            return Ok(())
-        }
+        Err(ref e) if e.kind() == io::ErrorKind::PermissionDenied => return Ok(()),
         Err(e) => {
             let cx = format!("failed to read directory `{}`", path.display());
             let e = CargoError::from(e);
-            return Err(e.context(cx).into())
+            return Err(e.context(cx).into());
         }
     };
     for dir in dirs {
@@ -104,13 +126,17 @@ fn has_manifest(path: &Path) -> bool {
     find_project_manifest_exact(path, "Cargo.toml").is_ok()
 }
 
-fn read_nested_packages(path: &Path,
-                        all_packages: &mut HashMap<PackageId, Package>,
-                        source_id: &SourceId,
-                        config: &Config,
-                        visited: &mut HashSet<PathBuf>,
-                        errors: &mut Vec<CargoError>) -> CargoResult<()> {
-    if !visited.insert(path.to_path_buf()) { return Ok(()) }
+fn read_nested_packages(
+    path: &Path,
+    all_packages: &mut HashMap<PackageId, Package>,
+    source_id: &SourceId,
+    config: &Config,
+    visited: &mut HashSet<PathBuf>,
+    errors: &mut Vec<CargoError>,
+) -> CargoResult<()> {
+    if !visited.insert(path.to_path_buf()) {
+        return Ok(());
+    }
 
     let manifest_path = find_project_manifest_exact(path, "Cargo.toml")?;
 
@@ -123,12 +149,14 @@ fn read_nested_packages(path: &Path,
             // it's safer to ignore malformed manifests to avoid
             //
             // TODO: Add a way to exclude folders?
-            info!("skipping malformed package found at `{}`",
-                  path.to_string_lossy());
-            errors.push(err);
+            info!(
+                "skipping malformed package found at `{}`",
+                path.to_string_lossy()
+            );
+            errors.push(err.into());
             return Ok(());
         }
-        Ok(tuple) => tuple
+        Ok(tuple) => tuple,
     };
 
     let manifest = match manifest {
@@ -138,11 +166,18 @@ fn read_nested_packages(path: &Path,
     let pkg = Package::new(manifest, &manifest_path);
 
     let pkg_id = pkg.package_id().clone();
-    if !all_packages.contains_key(&pkg_id) {
-        all_packages.insert(pkg_id, pkg);
-    } else {
-        info!("skipping nested package `{}` found at `{}`",
-              pkg.name(), path.to_string_lossy());
+    use std::collections::hash_map::Entry;
+    match all_packages.entry(pkg_id) {
+        Entry::Vacant(v) => {
+            v.insert(pkg);
+        }
+        Entry::Occupied(_) => {
+            info!(
+                "skipping nested package `{}` found at `{}`",
+                pkg.name(),
+                path.to_string_lossy()
+            );
+        }
     }
 
     // Registry sources are not allowed to have `path=` dependencies because
@@ -156,8 +191,7 @@ fn read_nested_packages(path: &Path,
     if !source_id.is_registry() {
         for p in nested.iter() {
             let path = util::normalize_path(&path.join(p));
-            read_nested_packages(&path, all_packages, source_id,
-                                      config, visited, errors)?;
+            read_nested_packages(&path, all_packages, source_id, config, visited, errors)?;
         }
     }
 
