@@ -9,6 +9,7 @@ use ignore::Match;
 use ignore::gitignore::GitignoreBuilder;
 
 use core::{Dependency, Package, PackageId, Source, SourceId, Summary};
+use core::source::MaybePackage;
 use ops;
 use util::{self, internal, CargoResult};
 use util::paths;
@@ -139,7 +140,7 @@ impl<'cfg> PathSource<'cfg> {
             .collect::<Result<Vec<_>, _>>()?;
 
         let glob_should_package = |relative_path: &Path| -> bool {
-            fn glob_match(patterns: &Vec<Pattern>, relative_path: &Path) -> bool {
+            fn glob_match(patterns: &[Pattern], relative_path: &Path) -> bool {
                 patterns
                     .iter()
                     .any(|pattern| pattern.matches_path(relative_path))
@@ -277,7 +278,7 @@ impl<'cfg> PathSource<'cfg> {
                     };
                     let path = util::without_prefix(root, cur).unwrap().join("Cargo.toml");
                     if index.get_path(&path, 0).is_some() {
-                        return Some(self.list_files_git(pkg, repo, filter));
+                        return Some(self.list_files_git(pkg, &repo, filter));
                     }
                 }
             }
@@ -296,7 +297,7 @@ impl<'cfg> PathSource<'cfg> {
     fn list_files_git(
         &self,
         pkg: &Package,
-        repo: git2::Repository,
+        repo: &git2::Repository,
         filter: &mut FnMut(&Path) -> CargoResult<bool>,
     ) -> CargoResult<Vec<PathBuf>> {
         warn!("list_files_git {}", pkg.package_id());
@@ -380,7 +381,7 @@ impl<'cfg> PathSource<'cfg> {
                 let rel = rel.replace(r"\", "/");
                 match repo.find_submodule(&rel).and_then(|s| s.open()) {
                     Ok(repo) => {
-                        let files = self.list_files_git(pkg, repo, filter)?;
+                        let files = self.list_files_git(pkg, &repo, filter)?;
                         ret.extend(files.into_iter());
                     }
                     Err(..) => {
@@ -447,10 +448,9 @@ impl<'cfg> PathSource<'cfg> {
         // TODO: Drop collect and sort after transition period and dropping warning tests.
         // See <https://github.com/rust-lang/cargo/issues/4268>
         // and <https://github.com/rust-lang/cargo/pull/4270>
-        let mut entries: Vec<fs::DirEntry> = fs::read_dir(path)?.map(|e| e.unwrap()).collect();
-        entries.sort_by(|a, b| a.path().as_os_str().cmp(b.path().as_os_str()));
-        for entry in entries {
-            let path = entry.path();
+        let mut entries: Vec<PathBuf> = fs::read_dir(path)?.map(|e| e.unwrap().path()).collect();
+        entries.sort_unstable_by(|a, b| a.as_os_str().cmp(b.as_os_str()));
+        for path in entries {
             let name = path.file_name().and_then(|s| s.to_str());
             // Skip dotfile directories
             if name.map(|s| s.starts_with('.')) == Some(true) {
@@ -481,7 +481,7 @@ impl<'cfg> PathSource<'cfg> {
             // condition where this path was rm'ed - either way,
             // we can ignore the error and treat the path's mtime
             // as 0.
-            let mtime = paths::mtime(&file).unwrap_or(FileTime::zero());
+            let mtime = paths::mtime(&file).unwrap_or_else(|_| FileTime::zero());
             if mtime > max {
                 max = mtime;
                 max_path = file;
@@ -490,6 +490,10 @@ impl<'cfg> PathSource<'cfg> {
         trace!("last modified file {}: {}", self.path.display(), max);
         Ok((max, max_path))
     }
+
+    pub fn path(&self) -> &Path {
+        &self.path
+    }
 }
 
 impl<'cfg> Debug for PathSource<'cfg> {
@@ -508,6 +512,13 @@ impl<'cfg> Source for PathSource<'cfg> {
         Ok(())
     }
 
+    fn fuzzy_query(&mut self, _dep: &Dependency, f: &mut FnMut(Summary)) -> CargoResult<()> {
+        for s in self.packages.iter().map(|p| p.summary()) {
+            f(s.clone())
+        }
+        Ok(())
+    }
+
     fn supports_checksums(&self) -> bool {
         false
     }
@@ -530,16 +541,28 @@ impl<'cfg> Source for PathSource<'cfg> {
         Ok(())
     }
 
-    fn download(&mut self, id: &PackageId) -> CargoResult<Package> {
+    fn download(&mut self, id: &PackageId) -> CargoResult<MaybePackage> {
         trace!("getting packages; id={}", id);
 
         let pkg = self.packages.iter().find(|pkg| pkg.package_id() == id);
         pkg.cloned()
+            .map(MaybePackage::Ready)
             .ok_or_else(|| internal(format!("failed to find {} in path source", id)))
     }
 
+    fn finish_download(&mut self, _id: &PackageId, _data: Vec<u8>) -> CargoResult<Package> {
+        panic!("no download should have started")
+    }
+
     fn fingerprint(&self, pkg: &Package) -> CargoResult<String> {
         let (max, max_path) = self.last_modified_file(pkg)?;
         Ok(format!("{} ({})", max, max_path.display()))
     }
+
+    fn describe(&self) -> String {
+        match self.source_id.url().to_file_path() {
+            Ok(path) => path.display().to_string(),
+            Err(_) => self.source_id.to_string(),
+        }
+    }
 }
