@@ -4,10 +4,9 @@ use std::path::Path;
 
 use core::PackageId;
 use hex;
-use sources::registry::{RegistryData, RegistryConfig};
-use util::FileLock;
+use sources::registry::{RegistryConfig, RegistryData, MaybeLock};
 use util::paths;
-use util::{Config, Sha256, Filesystem};
+use util::{Config, Filesystem, Sha256, FileLock};
 use util::errors::{CargoResult, CargoResultExt};
 
 pub struct LocalRegistry<'cfg> {
@@ -18,27 +17,31 @@ pub struct LocalRegistry<'cfg> {
 }
 
 impl<'cfg> LocalRegistry<'cfg> {
-    pub fn new(root: &Path,
-               config: &'cfg Config,
-               name: &str) -> LocalRegistry<'cfg> {
+    pub fn new(root: &Path, config: &'cfg Config, name: &str) -> LocalRegistry<'cfg> {
         LocalRegistry {
             src_path: config.registry_source_path().join(name),
             index_path: Filesystem::new(root.join("index")),
             root: Filesystem::new(root.to_path_buf()),
-            config: config,
+            config,
         }
     }
 }
 
 impl<'cfg> RegistryData for LocalRegistry<'cfg> {
+    fn prepare(&self) -> CargoResult<()> {
+        Ok(())
+    }
+
     fn index_path(&self) -> &Filesystem {
         &self.index_path
     }
 
-    fn load(&self,
-            root: &Path,
-            path: &Path,
-            data: &mut FnMut(&[u8]) -> CargoResult<()>) -> CargoResult<()> {
+    fn load(
+        &self,
+        root: &Path,
+        path: &Path,
+        data: &mut FnMut(&[u8]) -> CargoResult<()>,
+    ) -> CargoResult<()> {
         data(&paths::read_bytes(&root.join(path))?)
     }
 
@@ -54,29 +57,27 @@ impl<'cfg> RegistryData for LocalRegistry<'cfg> {
         // these directories exist.
         let root = self.root.clone().into_path_unlocked();
         if !root.is_dir() {
-            bail!("local registry path is not a directory: {}",
-                  root.display())
+            bail!("local registry path is not a directory: {}", root.display())
         }
         let index_path = self.index_path.clone().into_path_unlocked();
         if !index_path.is_dir() {
-            bail!("local registry index path is not a directory: {}",
-                  index_path.display())
+            bail!(
+                "local registry index path is not a directory: {}",
+                index_path.display()
+            )
         }
         Ok(())
     }
 
-    fn download(&mut self, pkg: &PackageId, checksum: &str)
-                -> CargoResult<FileLock> {
+    fn download(&mut self, pkg: &PackageId, checksum: &str) -> CargoResult<MaybeLock> {
         let crate_file = format!("{}-{}.crate", pkg.name(), pkg.version());
-        let mut crate_file = self.root.open_ro(&crate_file,
-                                               self.config,
-                                               "crate file")?;
+        let mut crate_file = self.root.open_ro(&crate_file, self.config, "crate file")?;
 
         // If we've already got an unpacked version of this crate, then skip the
         // checksum below as it is in theory already verified.
         let dst = format!("{}-{}", pkg.name(), pkg.version());
         if self.src_path.join(dst).into_path_unlocked().exists() {
-            return Ok(crate_file)
+            return Ok(MaybeLock::Ready(crate_file));
         }
 
         self.config.shell().status("Unpacking", pkg)?;
@@ -86,11 +87,11 @@ impl<'cfg> RegistryData for LocalRegistry<'cfg> {
         let mut state = Sha256::new();
         let mut buf = [0; 64 * 1024];
         loop {
-            let n = crate_file.read(&mut buf).chain_err(|| {
-                format!("failed to read `{}`", crate_file.path().display())
-            })?;
+            let n = crate_file
+                .read(&mut buf)
+                .chain_err(|| format!("failed to read `{}`", crate_file.path().display()))?;
             if n == 0 {
-                break
+                break;
             }
             state.update(&buf[..n]);
         }
@@ -100,6 +101,12 @@ impl<'cfg> RegistryData for LocalRegistry<'cfg> {
 
         crate_file.seek(SeekFrom::Start(0))?;
 
-        Ok(crate_file)
+        Ok(MaybeLock::Ready(crate_file))
+    }
+
+    fn finish_download(&mut self, _pkg: &PackageId, _checksum: &str, _data: &[u8])
+        -> CargoResult<FileLock>
+    {
+        panic!("this source doesn't download")
     }
 }
