@@ -40,10 +40,6 @@ pub enum Message {
         file: String,
         message: String,
     },
-    PreviewNotFound {
-        file: String,
-        edition: String,
-    },
     EditionAlreadyEnabled {
         file: String,
         edition: String,
@@ -81,7 +77,6 @@ impl Message {
 
 pub struct DiagnosticPrinter<'a> {
     config: &'a Config,
-    preview_not_found: HashSet<String>,
     edition_already_enabled: HashSet<String>,
     idiom_mismatch: HashSet<String>,
 }
@@ -90,7 +85,6 @@ impl<'a> DiagnosticPrinter<'a> {
     pub fn new(config: &'a Config) -> DiagnosticPrinter<'a> {
         DiagnosticPrinter {
             config,
-            preview_not_found: HashSet::new(),
             edition_already_enabled: HashSet::new(),
             idiom_mismatch: HashSet::new(),
         }
@@ -140,21 +134,6 @@ impl<'a> DiagnosticPrinter<'a> {
                 write!(self.config.shell().err(), "{}", PLEASE_REPORT_THIS_BUG)?;
                 Ok(())
             }
-            Message::PreviewNotFound { file, edition } => {
-                // By default we're fixing a lot of things concurrently, don't
-                // warn about the same file multiple times.
-                if !self.preview_not_found.insert(file.clone()) {
-                    return Ok(())
-                }
-                self.config.shell().warn(&format!(
-                    "failed to find `#![feature(rust_{}_preview)]` in `{}`\n\
-                     this may cause `cargo fix` to not be able to fix all\n\
-                     issues in preparation for the {0} edition",
-                    edition,
-                    file,
-                ))?;
-                Ok(())
-            }
             Message::EditionAlreadyEnabled { file, edition } => {
                 // Like above, only warn once per file
                 if !self.edition_already_enabled.insert(file.clone()) {
@@ -171,7 +150,7 @@ your `Cargo.toml` and then rerun this command. Once all warnings have been fixed
 then you can re-enable the `edition` key in `Cargo.toml`. For some more
 information about transitioning to the {0} edition see:
 
-  https://rust-lang-nursery.github.io/edition-guide/editions/transitioning-your-code-to-a-new-edition.html
+  https://rust-lang-nursery.github.io/edition-guide/editions/transitioning-an-existing-project-to-a-new-edition.html
 ",
                     edition,
                     file,
@@ -193,7 +172,7 @@ consider migrating to the {0} edition by adding `edition = '{0}'` to
 `Cargo.toml` and then rerunning this command; a more detailed transition
 guide can be found at
 
-  https://rust-lang-nursery.github.io/edition-guide/editions/transitioning-your-code-to-a-new-edition.html
+  https://rust-lang-nursery.github.io/edition-guide/editions/transitioning-an-existing-project-to-a-new-edition.html
 ",
                     idioms,
                     file,
