@@ -38,7 +38,7 @@ fn test_lev_distance() {
     use std::char::{from_u32, MAX};
     // Test bytelength agnosticity
     for c in (0u32..MAX as u32)
-        .filter_map(|i| from_u32(i))
+        .filter_map(from_u32)
         .map(|i| i.to_string())
     {
         assert_eq!(lev_distance(&c, &c), 0);
