@@ -8,7 +8,7 @@ use std::process::{Command, Output, Stdio};
 use jobserver::Client;
 use shell_escape::escape;
 
-use util::{process_error, CargoError, CargoResult, CargoResultExt, read2};
+use util::{process_error, CargoResult, CargoResultExt, read2};
 
 /// A builder object for an external process, similar to `std::process::Command`.
 #[derive(Clone, Debug)]
@@ -133,7 +133,7 @@ impl ProcessBuilder {
         let mut command = self.build_command();
         let exit = command.status().chain_err(|| {
             process_error(
-                &format!("could not execute process `{}`", self.debug_string()),
+                &format!("could not execute process {}", self),
                 None,
                 None,
             )
@@ -143,40 +143,30 @@ impl ProcessBuilder {
             Ok(())
         } else {
             Err(process_error(
-                &format!(
-                    "process didn't exit successfully: `{}`",
-                    self.debug_string()
-                ),
-                Some(&exit),
+                &format!("process didn't exit successfully: {}", self),
+                Some(exit),
                 None,
             ).into())
         }
     }
 
-    /// On unix, executes the process using the unix syscall `execvp`, which will block this
-    /// process, and will only return if there is an error. On windows this is a synonym for
-    /// `exec`.
-    #[cfg(unix)]
+    /// Replaces the current process with the target process.
+    ///
+    /// On Unix, this executes the process using the unix syscall `execvp`, which will block
+    /// this process, and will only return if there is an error.
+    ///
+    /// On Windows this isn't technically possible. Instead we emulate it to the best of our
+    /// ability. One aspect we fix here is that we specify a handler for the ctrl-c handler.
+    /// In doing so (and by effectively ignoring it) we should emulate proxying ctrl-c
+    /// handling to the application at hand, which will either terminate or handle it itself.
+    /// According to microsoft's documentation at:
+    /// https://docs.microsoft.com/en-us/windows/console/ctrl-c-and-ctrl-break-signals
+    /// the ctrl-c signal is sent to all processes attached to a terminal, which should
+    /// include our child process. If the child terminates then we'll reap them in Cargo
+    /// pretty quickly, and if the child handles the signal then we won't terminate
+    /// (and we shouldn't!) until the process itself later exits.
     pub fn exec_replace(&self) -> CargoResult<()> {
-        use std::os::unix::process::CommandExt;
-
-        let mut command = self.build_command();
-        let error = command.exec();
-        Err(CargoError::from(error)
-            .context(process_error(
-                &format!("could not execute process `{}`", self.debug_string()),
-                None,
-                None,
-            ))
-            .into())
-    }
-
-    /// On unix, executes the process using the unix syscall `execvp`, which will block this
-    /// process, and will only return if there is an error. On windows this is a synonym for
-    /// `exec`.
-    #[cfg(windows)]
-    pub fn exec_replace(&self) -> CargoResult<()> {
-        self.exec()
+        imp::exec_replace(self)
     }
 
     /// Execute the process, returning the stdio output, or an error if non-zero exit status.
@@ -185,7 +175,7 @@ impl ProcessBuilder {
 
         let output = command.output().chain_err(|| {
             process_error(
-                &format!("could not execute process `{}`", self.debug_string()),
+                &format!("could not execute process {}", self),
                 None,
                 None,
             )
@@ -195,11 +185,8 @@ impl ProcessBuilder {
             Ok(output)
         } else {
             Err(process_error(
-                &format!(
-                    "process didn't exit successfully: `{}`",
-                    self.debug_string()
-                ),
-                Some(&output.status),
+                &format!("process didn't exit successfully: {}", self),
+                Some(output.status),
                 Some(&output),
             ).into())
         }
@@ -261,7 +248,7 @@ impl ProcessBuilder {
         })()
             .chain_err(|| {
             process_error(
-                &format!("could not execute process `{}`", self.debug_string()),
+                &format!("could not execute process {}", self),
                 None,
                 None,
             )
@@ -276,20 +263,17 @@ impl ProcessBuilder {
             let to_print = if print_output { Some(&output) } else { None };
             if !output.status.success() {
                 return Err(process_error(
-                    &format!(
-                        "process didn't exit successfully: `{}`",
-                        self.debug_string()
-                    ),
-                    Some(&output.status),
+                    &format!("process didn't exit successfully: {}", self),
+                    Some(output.status),
                     to_print,
                 ).into());
             } else if let Some(e) = callback_error {
                 let cx = process_error(
-                    &format!("failed to parse process output: `{}`", self.debug_string()),
-                    Some(&output.status),
+                    &format!("failed to parse process output: {}", self),
+                    Some(output.status),
                     to_print,
                 );
-                return Err(CargoError::from(e).context(cx).into());
+                return Err(e.context(cx).into());
             }
         }
 
@@ -321,16 +305,6 @@ impl ProcessBuilder {
         }
         command
     }
-
-    /// Get the command line for the process as a string.
-    fn debug_string(&self) -> String {
-        let mut program = format!("{}", self.program.to_string_lossy());
-        for arg in &self.args {
-            program.push(' ');
-            program.push_str(&format!("{}", arg.to_string_lossy()));
-        }
-        program
-    }
 }
 
 /// A helper function to create a `ProcessBuilder`.
@@ -343,3 +317,51 @@ pub fn process<T: AsRef<OsStr>>(cmd: T) -> ProcessBuilder {
         jobserver: None,
     }
 }
+
+#[cfg(unix)]
+mod imp {
+    use CargoResult;
+    use std::os::unix::process::CommandExt;
+    use util::{process_error, ProcessBuilder};
+
+    pub fn exec_replace(process_builder: &ProcessBuilder) -> CargoResult<()> {
+        let mut command = process_builder.build_command();
+        let error = command.exec();
+        Err(::util::CargoError::from(error)
+            .context(process_error(
+                &format!("could not execute process {}", process_builder),
+                None,
+                None,
+            ))
+            .into())
+    }
+}
+
+#[cfg(windows)]
+mod imp {
+    extern crate winapi;
+
+    use CargoResult;
+    use util::{process_error, ProcessBuilder};
+    use self::winapi::shared::minwindef::{BOOL, DWORD, FALSE, TRUE};
+    use self::winapi::um::consoleapi::SetConsoleCtrlHandler;
+
+    unsafe extern "system" fn ctrlc_handler(_: DWORD) -> BOOL {
+        // Do nothing. Let the child process handle it.
+        TRUE
+    }
+
+    pub fn exec_replace(process_builder: &ProcessBuilder) -> CargoResult<()> {
+        unsafe {
+            if SetConsoleCtrlHandler(Some(ctrlc_handler), TRUE) == FALSE {
+                return Err(process_error(
+                    "Could not set Ctrl-C handler.",
+                    None,
+                    None).into());
+            }
+        }
+
+        // Just exec the process as normal.
+        process_builder.exec()
+    }
+}
