@@ -14,17 +14,21 @@ use std::path::{Path, PathBuf};
 use std::fs::{self, DirEntry};
 use std::collections::HashSet;
 
-use core::{compiler, Edition, Target};
-use util::errors::CargoResult;
-use super::{LibKind, PathValue, StringOrBool, TomlBenchTarget, TomlBinTarget, TomlExampleTarget,
-            TomlLibTarget, TomlManifest, TomlTarget, TomlTestTarget};
+use core::{compiler, Edition, Feature, Features, Target};
+use util::errors::{CargoResult, CargoResultExt};
+use super::{
+    LibKind, PathValue, StringOrBool, StringOrVec, TomlBenchTarget, TomlBinTarget,
+    TomlExampleTarget, TomlLibTarget, TomlManifest, TomlTarget, TomlTestTarget,
+};
 
 pub fn targets(
+    features: &Features,
     manifest: &TomlManifest,
     package_name: &str,
     package_root: &Path,
     edition: Edition,
     custom_build: &Option<StringOrBool>,
+    metabuild: &Option<StringOrVec>,
     warnings: &mut Vec<String>,
     errors: &mut Vec<String>,
 ) -> CargoResult<Vec<Target>> {
@@ -33,6 +37,7 @@ pub fn targets(
     let has_lib;
 
     if let Some(target) = clean_lib(
+        features,
         manifest.lib.as_ref(),
         package_root,
         package_name,
@@ -52,6 +57,7 @@ pub fn targets(
         .ok_or_else(|| format_err!("manifest has no `package` (or `project`)"))?;
 
     targets.extend(clean_bins(
+        features,
         manifest.bin.as_ref(),
         package_root,
         package_name,
@@ -63,6 +69,7 @@ pub fn targets(
     )?);
 
     targets.extend(clean_examples(
+        features,
         manifest.example.as_ref(),
         package_root,
         edition,
@@ -72,6 +79,7 @@ pub fn targets(
     )?);
 
     targets.extend(clean_tests(
+        features,
         manifest.test.as_ref(),
         package_root,
         edition,
@@ -81,6 +89,7 @@ pub fn targets(
     )?);
 
     targets.extend(clean_benches(
+        features,
         manifest.bench.as_ref(),
         package_root,
         edition,
@@ -91,6 +100,9 @@ pub fn targets(
 
     // processing the custom build script
     if let Some(custom_build) = manifest.maybe_custom_build(custom_build, package_root) {
+        if metabuild.is_some() {
+            bail!("cannot specify both `metabuild` and `build`");
+        }
         let name = format!(
             "build-script-{}",
             custom_build
@@ -101,13 +113,32 @@ pub fn targets(
         targets.push(Target::custom_build_target(
             &name,
             package_root.join(custom_build),
+            edition,
         ));
     }
+    if let Some(metabuild) = metabuild {
+        // Verify names match available build deps.
+        let bdeps = manifest.build_dependencies.as_ref();
+        for name in &metabuild.0 {
+            if !bdeps.map_or(false, |bd| bd.contains_key(name)) {
+                bail!(
+                    "metabuild package `{}` must be specified in `build-dependencies`",
+                    name
+                );
+            }
+        }
+
+        targets.push(Target::metabuild_target(&format!(
+            "metabuild-{}",
+            package.name
+        )));
+    }
 
     Ok(targets)
 }
 
 fn clean_lib(
+    features: &Features,
     toml_lib: Option<&TomlLibTarget>,
     package_root: &Path,
     package_name: &str,
@@ -176,18 +207,19 @@ fn clean_lib(
     // both at once.
     let crate_types = match (lib.crate_types(), lib.plugin, lib.proc_macro()) {
         (_, Some(true), Some(true)) => bail!("lib.plugin and lib.proc-macro cannot both be true"),
-        (Some(kinds), _, _) => kinds.iter().map(|s| LibKind::from_str(s)).collect(),
+        (Some(kinds), _, _) => kinds.iter().map(|s| s.into()).collect(),
         (None, Some(true), _) => vec![LibKind::Dylib],
         (None, _, Some(true)) => vec![LibKind::ProcMacro],
         (None, _, _) => vec![LibKind::Lib],
     };
 
-    let mut target = Target::lib_target(&lib.name(), crate_types, path);
-    configure(lib, &mut target);
+    let mut target = Target::lib_target(&lib.name(), crate_types, path, edition);
+    configure(features, lib, &mut target)?;
     Ok(Some(target))
 }
 
 fn clean_bins(
+    features: &Features,
     toml_bins: Option<&Vec<TomlBinTarget>>,
     package_root: &Path,
     package_name: &str,
@@ -262,8 +294,13 @@ fn clean_bins(
             Err(e) => bail!("{}", e),
         };
 
-        let mut target = Target::bin_target(&bin.name(), path, bin.required_features.clone());
-        configure(bin, &mut target);
+        let mut target = Target::bin_target(
+            &bin.name(),
+            path,
+            bin.required_features.clone(),
+            edition,
+        );
+        configure(features, bin, &mut target)?;
         result.push(target);
     }
     return Ok(result);
@@ -289,6 +326,7 @@ fn clean_bins(
 }
 
 fn clean_examples(
+    features: &Features,
     toml_examples: Option<&Vec<TomlExampleTarget>>,
     package_root: &Path,
     edition: Edition,
@@ -314,7 +352,7 @@ fn clean_examples(
     let mut result = Vec::new();
     for (path, toml) in targets {
         let crate_types = match toml.crate_types() {
-            Some(kinds) => kinds.iter().map(|s| LibKind::from_str(s)).collect(),
+            Some(kinds) => kinds.iter().map(|s| s.into()).collect(),
             None => Vec::new(),
         };
 
@@ -323,8 +361,9 @@ fn clean_examples(
             crate_types,
             path,
             toml.required_features.clone(),
+            edition,
         );
-        configure(&toml, &mut target);
+        configure(features, &toml, &mut target)?;
         result.push(target);
     }
 
@@ -332,6 +371,7 @@ fn clean_examples(
 }
 
 fn clean_tests(
+    features: &Features,
     toml_tests: Option<&Vec<TomlTestTarget>>,
     package_root: &Path,
     edition: Edition,
@@ -356,14 +396,20 @@ fn clean_tests(
 
     let mut result = Vec::new();
     for (path, toml) in targets {
-        let mut target = Target::test_target(&toml.name(), path, toml.required_features.clone());
-        configure(&toml, &mut target);
+        let mut target = Target::test_target(
+            &toml.name(),
+            path,
+            toml.required_features.clone(),
+            edition,
+        );
+        configure(features, &toml, &mut target)?;
         result.push(target);
     }
     Ok(result)
 }
 
 fn clean_benches(
+    features: &Features,
     toml_benches: Option<&Vec<TomlBenchTarget>>,
     package_root: &Path,
     edition: Edition,
@@ -409,8 +455,13 @@ fn clean_benches(
 
     let mut result = Vec::new();
     for (path, toml) in targets {
-        let mut target = Target::bench_target(&toml.name(), path, toml.required_features.clone());
-        configure(&toml, &mut target);
+        let mut target = Target::bench_target(
+            &toml.name(),
+            path,
+            toml.required_features.clone(),
+            edition,
+        );
+        configure(features, &toml, &mut target)?;
         result.push(target);
     }
 
@@ -682,7 +733,11 @@ fn validate_unique_names(targets: &[TomlTarget], target_kind: &str) -> CargoResu
     Ok(())
 }
 
-fn configure(toml: &TomlTarget, target: &mut Target) {
+fn configure(
+    features: &Features,
+    toml: &TomlTarget,
+    target: &mut Target,
+) -> CargoResult<()> {
     let t2 = target.clone();
     target
         .set_tested(toml.test.unwrap_or_else(|| t2.tested()))
@@ -695,6 +750,11 @@ fn configure(toml: &TomlTarget, target: &mut Target) {
             (Some(true), _) | (_, Some(true)) => true,
             (Some(false), _) | (_, Some(false)) => false,
         });
+    if let Some(edition) = toml.edition.clone() {
+        features.require(Feature::edition()).chain_err(|| "editions are unstable")?;
+        target.set_edition(edition.parse().chain_err(|| "failed to parse the `edition` key")?);
+    }
+    Ok(())
 }
 
 fn target_path(
