@@ -1,4 +1,5 @@
 #![allow(unknown_lints)]
+#![cfg_attr(feature = "cargo-clippy", allow(identity_op))] // used for vertical alignment
 
 extern crate curl;
 #[macro_use]
@@ -67,7 +68,10 @@ pub struct NewCrateDependency {
     pub version_req: String,
     pub target: Option<String>,
     pub kind: String,
-    #[serde(skip_serializing_if = "Option::is_none")] pub registry: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub registry: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub explicit_name_in_toml: Option<String>,
 }
 
 #[derive(Deserialize)]
@@ -133,20 +137,20 @@ impl Registry {
 
     pub fn add_owners(&mut self, krate: &str, owners: &[&str]) -> Result<String> {
         let body = serde_json::to_string(&OwnersReq { users: owners })?;
-        let body = self.put(format!("/crates/{}/owners", krate), body.as_bytes())?;
+        let body = self.put(&format!("/crates/{}/owners", krate), body.as_bytes())?;
         assert!(serde_json::from_str::<OwnerResponse>(&body)?.ok);
         Ok(serde_json::from_str::<OwnerResponse>(&body)?.msg)
     }
 
     pub fn remove_owners(&mut self, krate: &str, owners: &[&str]) -> Result<()> {
         let body = serde_json::to_string(&OwnersReq { users: owners })?;
-        let body = self.delete(format!("/crates/{}/owners", krate), Some(body.as_bytes()))?;
+        let body = self.delete(&format!("/crates/{}/owners", krate), Some(body.as_bytes()))?;
         assert!(serde_json::from_str::<OwnerResponse>(&body)?.ok);
         Ok(())
     }
 
     pub fn list_owners(&mut self, krate: &str) -> Result<Vec<User>> {
-        let body = self.get(format!("/crates/{}/owners", krate))?;
+        let body = self.get(&format!("/crates/{}/owners", krate))?;
         Ok(serde_json::from_str::<Users>(&body)?.users)
     }
 
@@ -167,18 +171,16 @@ impl Registry {
                     (json.len() >> 8) as u8,
                     (json.len() >> 16) as u8,
                     (json.len() >> 24) as u8,
-                ].iter()
-                    .map(|x| *x),
+                ].iter().cloned(),
             );
-            w.extend(json.as_bytes().iter().map(|x| *x));
+            w.extend(json.as_bytes().iter().cloned());
             w.extend(
                 [
                     (stat.len() >> 0) as u8,
                     (stat.len() >> 8) as u8,
                     (stat.len() >> 16) as u8,
                     (stat.len() >> 24) as u8,
-                ].iter()
-                    .map(|x| *x),
+                ].iter().cloned(),
             );
             w
         };
@@ -201,10 +203,10 @@ impl Registry {
 
         let body = handle(&mut self.handle, &mut |buf| body.read(buf).unwrap_or(0))?;
 
-        let response = if body.len() > 0 {
-            body.parse::<serde_json::Value>()?
-        } else {
+        let response = if body.is_empty() {
             "{}".parse()?
+        } else {
+            body.parse::<serde_json::Value>()?
         };
 
         let invalid_categories: Vec<String> = response
@@ -230,7 +232,7 @@ impl Registry {
     pub fn search(&mut self, query: &str, limit: u32) -> Result<(Vec<Crate>, u32)> {
         let formatted_query = percent_encode(query.as_bytes(), QUERY_ENCODE_SET);
         let body = self.req(
-            format!("/crates?q={}&per_page={}", formatted_query, limit),
+            &format!("/crates?q={}&per_page={}", formatted_query, limit),
             None,
             Auth::Unauthorized,
         )?;
@@ -240,33 +242,33 @@ impl Registry {
     }
 
     pub fn yank(&mut self, krate: &str, version: &str) -> Result<()> {
-        let body = self.delete(format!("/crates/{}/{}/yank", krate, version), None)?;
+        let body = self.delete(&format!("/crates/{}/{}/yank", krate, version), None)?;
         assert!(serde_json::from_str::<R>(&body)?.ok);
         Ok(())
     }
 
     pub fn unyank(&mut self, krate: &str, version: &str) -> Result<()> {
-        let body = self.put(format!("/crates/{}/{}/unyank", krate, version), &[])?;
+        let body = self.put(&format!("/crates/{}/{}/unyank", krate, version), &[])?;
         assert!(serde_json::from_str::<R>(&body)?.ok);
         Ok(())
     }
 
-    fn put(&mut self, path: String, b: &[u8]) -> Result<String> {
+    fn put(&mut self, path: &str, b: &[u8]) -> Result<String> {
         self.handle.put(true)?;
         self.req(path, Some(b), Auth::Authorized)
     }
 
-    fn get(&mut self, path: String) -> Result<String> {
+    fn get(&mut self, path: &str) -> Result<String> {
         self.handle.get(true)?;
         self.req(path, None, Auth::Authorized)
     }
 
-    fn delete(&mut self, path: String, b: Option<&[u8]>) -> Result<String> {
+    fn delete(&mut self, path: &str, b: Option<&[u8]>) -> Result<String> {
         self.handle.custom_request("DELETE")?;
         self.req(path, b, Auth::Authorized)
     }
 
-    fn req(&mut self, path: String, body: Option<&[u8]>, authorized: Auth) -> Result<String> {
+    fn req(&mut self, path: &str, body: Option<&[u8]>, authorized: Auth) -> Result<String> {
         self.handle.url(&format!("{}/api/v1{}", self.host, path))?;
         let mut headers = List::new();
         headers.append("Accept: application/json")?;
@@ -329,12 +331,9 @@ fn handle(handle: &mut Easy, read: &mut FnMut(&mut [u8]) -> usize) -> Result<Str
         Ok(body) => body,
         Err(..) => bail!("response body was not valid utf-8"),
     };
-    match serde_json::from_str::<ApiErrorList>(&body) {
-        Ok(errors) => {
-            let errors = errors.errors.into_iter().map(|s| s.detail);
-            bail!("api errors: {}", errors.collect::<Vec<_>>().join(", "))
-        }
-        Err(..) => {}
+    if let Ok(errors) = serde_json::from_str::<ApiErrorList>(&body) {
+        let errors = errors.errors.into_iter().map(|s| s.detail);
+        bail!("api errors: {}", errors.collect::<Vec<_>>().join(", "));
     }
     Ok(body)
 }
