@@ -1,5 +1,4 @@
-use support::{basic_bin_manifest, execs, project};
-use support::hamcrest::assert_that;
+use support::{basic_bin_manifest, project};
 
 #[test]
 fn alias_incorrect_config_type() {
@@ -12,17 +11,15 @@ fn alias_incorrect_config_type() {
             [alias]
             b-cargo-test = 5
         "#,
-        )
-        .build();
+        ).build();
 
-    assert_that(
-        p.cargo("b-cargo-test").arg("-v"),
-        execs().with_status(101).with_stderr_contains(
+    p.cargo("b-cargo-test -v")
+        .with_status(101)
+        .with_stderr_contains(
             "\
 [ERROR] invalid configuration for key `alias.b-cargo-test`
 expected a list, but found a integer for [..]",
-        ),
-    );
+        ).run();
 }
 
 #[test]
@@ -36,14 +33,11 @@ fn alias_default_config_overrides_config() {
             [alias]
             b = "not_build"
         "#,
-        )
-        .build();
+        ).build();
 
-    assert_that(
-        p.cargo("b").arg("-v"),
-        execs()
-            .with_stderr_contains("[COMPILING] foo v0.5.0 [..]"),
-    );
+    p.cargo("b -v")
+        .with_stderr_contains("[COMPILING] foo v0.5.0 [..]")
+        .run();
 }
 
 #[test]
@@ -57,18 +51,14 @@ fn alias_config() {
             [alias]
             b-cargo-test = "build"
         "#,
-        )
-        .build();
+        ).build();
 
-    assert_that(
-        p.cargo("b-cargo-test -v"),
-        execs()
-            .with_stderr_contains(
-                "\
+    p.cargo("b-cargo-test -v")
+        .with_stderr_contains(
+            "\
 [COMPILING] foo v0.5.0 [..]
 [RUNNING] `rustc --crate-name foo [..]",
-            ),
-    );
+        ).run();
 }
 
 #[test]
@@ -83,17 +73,14 @@ fn recursive_alias() {
             b-cargo-test = "build"
             a-cargo-test = ["b-cargo-test", "-v"]
         "#,
-        )
-        .build();
+        ).build();
 
-    assert_that(
-        p.cargo("a-cargo-test"),
-        execs().with_stderr_contains(
+    p.cargo("a-cargo-test")
+        .with_stderr_contains(
             "\
 [COMPILING] foo v0.5.0 [..]
 [RUNNING] `rustc --crate-name foo [..]",
-        ),
-    );
+        ).run();
 }
 
 #[test]
@@ -107,15 +94,12 @@ fn alias_list_test() {
             [alias]
             b-cargo-test = ["build", "--release"]
          "#,
-        )
-        .build();
+        ).build();
 
-    assert_that(
-        p.cargo("b-cargo-test").arg("-v"),
-        execs()
-            .with_stderr_contains("[COMPILING] foo v0.5.0 [..]")
-            .with_stderr_contains("[RUNNING] `rustc --crate-name [..]"),
-    );
+    p.cargo("b-cargo-test -v")
+        .with_stderr_contains("[COMPILING] foo v0.5.0 [..]")
+        .with_stderr_contains("[RUNNING] `rustc --crate-name [..]")
+        .run();
 }
 
 #[test]
@@ -129,15 +113,12 @@ fn alias_with_flags_config() {
             [alias]
             b-cargo-test = "build --release"
          "#,
-        )
-        .build();
+        ).build();
 
-    assert_that(
-        p.cargo("b-cargo-test").arg("-v"),
-        execs()
-            .with_stderr_contains("[COMPILING] foo v0.5.0 [..]")
-            .with_stderr_contains("[RUNNING] `rustc --crate-name foo [..]"),
-    );
+    p.cargo("b-cargo-test -v")
+        .with_stderr_contains("[COMPILING] foo v0.5.0 [..]")
+        .with_stderr_contains("[RUNNING] `rustc --crate-name foo [..]")
+        .run();
 }
 
 #[test]
@@ -151,17 +132,14 @@ fn cant_shadow_builtin() {
             [alias]
             build = "fetch"
          "#,
-        )
-        .build();
+        ).build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(
+    p.cargo("build")
+        .with_stderr(
             "\
 [WARNING] alias `build` is ignored, because it is shadowed by a built in command
 [COMPILING] foo v0.5.0 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ),
-    );
+        ).run();
 }
