@@ -1,10 +1,8 @@
-use cargotest::ChannelChanger;
-use cargotest::support::{execs, project, publish};
-use hamcrest::assert_that;
+use support::{project, publish};
 
 #[test]
 fn feature_required() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -14,12 +12,12 @@ fn feature_required() {
             authors = []
             im-a-teapot = true
         "#,
-        )
-        .file("src/lib.rs", "")
+        ).file("src/lib.rs", "")
         .build();
-    assert_that(
-        p.cargo("build").masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr(
             "\
 error: failed to parse manifest at `[..]`
 
@@ -31,12 +29,11 @@ Caused by:
 
 consider adding `cargo-features = [\"test-dummy-unstable\"]` to the manifest
 ",
-        ),
-    );
+        ).run();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 error: failed to parse manifest at `[..]`
 
@@ -50,13 +47,12 @@ this Cargo does not support nightly features, but if you
 switch to nightly channel you can add
 `cargo-features = [\"test-dummy-unstable\"]` to enable this feature
 ",
-        ),
-    );
+        ).run();
 }
 
 #[test]
 fn unknown_feature() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -67,25 +63,23 @@ fn unknown_feature() {
             version = "0.0.1"
             authors = []
         "#,
-        )
-        .file("src/lib.rs", "")
+        ).file("src/lib.rs", "")
         .build();
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 error: failed to parse manifest at `[..]`
 
 Caused by:
   unknown cargo feature `foo`
 ",
-        ),
-    );
+        ).run();
 }
 
 #[test]
 fn stable_feature_warns() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -96,25 +90,22 @@ fn stable_feature_warns() {
             version = "0.0.1"
             authors = []
         "#,
-        )
-        .file("src/lib.rs", "")
+        ).file("src/lib.rs", "")
         .build();
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(0).with_stderr(
+    p.cargo("build")
+        .with_stderr(
             "\
 warning: the cargo feature `test-dummy-stable` is now stable and is no longer \
 necessary to be listed in the manifest
 [COMPILING] a [..]
 [FINISHED] [..]
 ",
-        ),
-    );
+        ).run();
 }
 
 #[test]
 fn nightly_feature_requires_nightly() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -126,22 +117,20 @@ fn nightly_feature_requires_nightly() {
             authors = []
             im-a-teapot = true
         "#,
-        )
-        .file("src/lib.rs", "")
+        ).file("src/lib.rs", "")
         .build();
-    assert_that(
-        p.cargo("build").masquerade_as_nightly_cargo(),
-        execs().with_status(0).with_stderr(
+    p.cargo("build")
+        .masquerade_as_nightly_cargo()
+        .with_stderr(
             "\
 [COMPILING] a [..]
 [FINISHED] [..]
 ",
-        ),
-    );
+        ).run();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 error: failed to parse manifest at `[..]`
 
@@ -149,13 +138,12 @@ Caused by:
   the cargo feature `test-dummy-unstable` requires a nightly version of Cargo, \
   but this is the `stable` channel
 ",
-        ),
-    );
+        ).run();
 }
 
 #[test]
 fn nightly_feature_requires_nightly_in_dep() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -167,8 +155,7 @@ fn nightly_feature_requires_nightly_in_dep() {
             [dependencies]
             a = { path = "a" }
         "#,
-        )
-        .file("src/lib.rs", "")
+        ).file("src/lib.rs", "")
         .file(
             "a/Cargo.toml",
             r#"
@@ -180,23 +167,21 @@ fn nightly_feature_requires_nightly_in_dep() {
             authors = []
             im-a-teapot = true
         "#,
-        )
-        .file("a/src/lib.rs", "")
+        ).file("a/src/lib.rs", "")
         .build();
-    assert_that(
-        p.cargo("build").masquerade_as_nightly_cargo(),
-        execs().with_status(0).with_stderr(
+    p.cargo("build")
+        .masquerade_as_nightly_cargo()
+        .with_stderr(
             "\
 [COMPILING] a [..]
 [COMPILING] b [..]
 [FINISHED] [..]
 ",
-        ),
-    );
+        ).run();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 error: failed to load source for a dependency on `a`
 
@@ -210,13 +195,12 @@ Caused by:
   the cargo feature `test-dummy-unstable` requires a nightly version of Cargo, \
   but this is the `stable` channel
 ",
-        ),
-    );
+        ).run();
 }
 
 #[test]
 fn cant_publish() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -228,22 +212,20 @@ fn cant_publish() {
             authors = []
             im-a-teapot = true
         "#,
-        )
-        .file("src/lib.rs", "")
+        ).file("src/lib.rs", "")
         .build();
-    assert_that(
-        p.cargo("build").masquerade_as_nightly_cargo(),
-        execs().with_status(0).with_stderr(
+    p.cargo("build")
+        .masquerade_as_nightly_cargo()
+        .with_stderr(
             "\
 [COMPILING] a [..]
 [FINISHED] [..]
 ",
-        ),
-    );
+        ).run();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 error: failed to parse manifest at `[..]`
 
@@ -251,13 +233,12 @@ Caused by:
   the cargo feature `test-dummy-unstable` requires a nightly version of Cargo, \
   but this is the `stable` channel
 ",
-        ),
-    );
+        ).run();
 }
 
 #[test]
 fn z_flags_rejected() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -269,44 +250,35 @@ fn z_flags_rejected() {
             authors = []
             im-a-teapot = true
         "#,
-        )
-        .file("src/lib.rs", "")
+        ).file("src/lib.rs", "")
         .build();
-    assert_that(
-        p.cargo("build").arg("-Zprint-im-a-teapot"),
-        execs()
-            .with_status(101)
-            .with_stderr("error: the `-Z` flag is only accepted on the nightly channel of Cargo"),
-    );
+    p.cargo("build -Zprint-im-a-teapot")
+        .with_status(101)
+        .with_stderr("error: the `-Z` flag is only accepted on the nightly channel of Cargo")
+        .run();
 
-    assert_that(
-        p.cargo("build").masquerade_as_nightly_cargo().arg("-Zarg"),
-        execs()
-            .with_status(101)
-            .with_stderr("error: unknown `-Z` flag specified: arg"),
-    );
+    p.cargo("build -Zarg")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr("error: unknown `-Z` flag specified: arg")
+        .run();
 
-    assert_that(
-        p.cargo("build")
-            .masquerade_as_nightly_cargo()
-            .arg("-Zprint-im-a-teapot"),
-        execs()
-            .with_status(0)
-            .with_stdout("im-a-teapot = true\n")
-            .with_stderr(
-                "\
+    p.cargo("build -Zprint-im-a-teapot")
+        .masquerade_as_nightly_cargo()
+        .with_stdout("im-a-teapot = true\n")
+        .with_stderr(
+            "\
 [COMPILING] a [..]
 [FINISHED] [..]
 ",
-            ),
-    );
+        ).run();
 }
 
 #[test]
 fn publish_allowed() {
     publish::setup();
 
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -317,14 +289,10 @@ fn publish_allowed() {
             version = "0.0.1"
             authors = []
         "#,
-        )
-        .file("src/lib.rs", "")
+        ).file("src/lib.rs", "")
         .build();
-    assert_that(
-        p.cargo("publish")
-            .arg("--index")
-            .arg(publish::registry().to_string())
-            .masquerade_as_nightly_cargo(),
-        execs().with_status(0),
-    );
+    p.cargo("publish --index")
+        .arg(publish::registry().to_string())
+        .masquerade_as_nightly_cargo()
+        .run();
 }
