@@ -1,10 +1,9 @@
 use std::fs::File;
-use std::path::PathBuf;
 use std::io::prelude::*;
+use std::path::PathBuf;
 
-use cargotest::support::{cross_compile, execs, project, publish};
-use hamcrest::{assert_that, contains};
 use flate2::read::GzDecoder;
+use support::{cross_compile, project, publish};
 use tar::Archive;
 
 #[test]
@@ -13,7 +12,7 @@ fn simple_cross_package() {
         return;
     }
 
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -25,8 +24,7 @@ fn simple_cross_package() {
             description = "foo"
             repository = "bar"
         "#,
-        )
-        .file(
+        ).file(
             "src/main.rs",
             &format!(
                 r#"
@@ -37,22 +35,19 @@ fn simple_cross_package() {
         "#,
                 cross_compile::alternate_arch()
             ),
-        )
-        .build();
+        ).build();
 
     let target = cross_compile::alternate();
 
-    assert_that(
-        p.cargo("package").arg("--target").arg(&target),
-        execs().with_status(0).with_status(0).with_stderr(&format!(
-            "   Packaging foo v0.0.0 ({dir})
-   Verifying foo v0.0.0 ({dir})
-   Compiling foo v0.0.0 ({dir}/target/package/foo-0.0.0)
+    p.cargo("package --target")
+        .arg(&target)
+        .with_stderr(
+            "   Packaging foo v0.0.0 ([CWD])
+   Verifying foo v0.0.0 ([CWD])
+   Compiling foo v0.0.0 ([CWD]/target/package/foo-0.0.0)
     Finished dev [unoptimized + debuginfo] target(s) in [..]
 ",
-            dir = p.url()
-        )),
-    );
+        ).run();
 
     // Check that the tarball contains the files
     let f = File::open(&p.root().join("target/package/foo-0.0.0.crate")).unwrap();
@@ -64,18 +59,9 @@ fn simple_cross_package() {
     let entry_paths = entries
         .map(|entry| entry.unwrap().path().unwrap().into_owned())
         .collect::<Vec<PathBuf>>();
-    assert_that(
-        &entry_paths,
-        contains(vec![PathBuf::from("foo-0.0.0/Cargo.toml")]),
-    );
-    assert_that(
-        &entry_paths,
-        contains(vec![PathBuf::from("foo-0.0.0/Cargo.toml.orig")]),
-    );
-    assert_that(
-        &entry_paths,
-        contains(vec![PathBuf::from("foo-0.0.0/src/main.rs")]),
-    );
+    assert!(entry_paths.contains(&PathBuf::from("foo-0.0.0/Cargo.toml")));
+    assert!(entry_paths.contains(&PathBuf::from("foo-0.0.0/Cargo.toml.orig")));
+    assert!(entry_paths.contains(&PathBuf::from("foo-0.0.0/src/main.rs")));
 }
 
 #[test]
@@ -86,7 +72,7 @@ fn publish_with_target() {
 
     publish::setup();
 
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -98,8 +84,7 @@ fn publish_with_target() {
             description = "foo"
             repository = "bar"
         "#,
-        )
-        .file(
+        ).file(
             "src/main.rs",
             &format!(
                 r#"
@@ -110,27 +95,22 @@ fn publish_with_target() {
         "#,
                 cross_compile::alternate_arch()
             ),
-        )
-        .build();
+        ).build();
 
     let target = cross_compile::alternate();
 
-    assert_that(
-        p.cargo("publish")
-            .arg("--index")
-            .arg(publish::registry().to_string())
-            .arg("--target")
-            .arg(&target),
-        execs().with_status(0).with_stderr(&format!(
-            "    Updating registry `{registry}`
-   Packaging foo v0.0.0 ({dir})
-   Verifying foo v0.0.0 ({dir})
-   Compiling foo v0.0.0 ({dir}/target/package/foo-0.0.0)
+    p.cargo("publish --index")
+        .arg(publish::registry().to_string())
+        .arg("--target")
+        .arg(&target)
+        .with_stderr(&format!(
+            "    Updating `{registry}` index
+   Packaging foo v0.0.0 ([CWD])
+   Verifying foo v0.0.0 ([CWD])
+   Compiling foo v0.0.0 ([CWD]/target/package/foo-0.0.0)
     Finished dev [unoptimized + debuginfo] target(s) in [..]
-   Uploading foo v0.0.0 ({dir})
+   Uploading foo v0.0.0 ([CWD])
 ",
-            dir = p.url(),
-            registry = publish::registry()
-        )),
-    );
+            registry = publish::registry_path().to_str().unwrap()
+        )).run();
 }
