@@ -1,9 +1,8 @@
-use std::fs;
 use std::env;
+use std::fs;
 
+use support::{basic_manifest, project};
 use support::{is_nightly, rustc_host};
-use support::{basic_manifest, execs, project};
-use support::hamcrest::assert_that;
 
 #[test]
 fn plugin_to_the_max() {
@@ -26,8 +25,7 @@ fn plugin_to_the_max() {
             [dependencies.bar]
             path = "../bar"
         "#,
-        )
-        .file(
+        ).file(
             "src/main.rs",
             r#"
             #![feature(plugin)]
@@ -36,8 +34,7 @@ fn plugin_to_the_max() {
 
             fn main() { foo_lib::foo(); }
         "#,
-        )
-        .file(
+        ).file(
             "src/foo_lib.rs",
             r#"
             #![feature(plugin)]
@@ -45,9 +42,9 @@ fn plugin_to_the_max() {
 
             pub fn foo() {}
         "#,
-        )
-        .build();
-    let _bar = project().at("bar")
+        ).build();
+    let _bar = project()
+        .at("bar")
         .file(
             "Cargo.toml",
             r#"
@@ -63,8 +60,7 @@ fn plugin_to_the_max() {
             [dependencies.baz]
             path = "../baz"
         "#,
-        )
-        .file(
+        ).file(
             "src/lib.rs",
             r#"
             #![feature(plugin_registrar, rustc_private)]
@@ -79,9 +75,9 @@ fn plugin_to_the_max() {
                 println!("{}", baz::baz());
             }
         "#,
-        )
-        .build();
-    let _baz = project().at("baz")
+        ).build();
+    let _baz = project()
+        .at("baz")
         .file(
             "Cargo.toml",
             r#"
@@ -94,12 +90,11 @@ fn plugin_to_the_max() {
             name = "baz"
             crate_type = ["dylib"]
         "#,
-        )
-        .file("src/lib.rs", "pub fn baz() -> i32 { 1 }")
+        ).file("src/lib.rs", "pub fn baz() -> i32 { 1 }")
         .build();
 
-    assert_that(foo.cargo("build"), execs());
-    assert_that(foo.cargo("doc"), execs());
+    foo.cargo("build").run();
+    foo.cargo("doc").run();
 }
 
 #[test]
@@ -108,17 +103,18 @@ fn plugin_with_dynamic_native_dependency() {
         return;
     }
 
-    let workspace = project().at("ws")
+    let workspace = project()
+        .at("ws")
         .file(
             "Cargo.toml",
             r#"
             [workspace]
             members = ["builder", "foo"]
         "#,
-        )
-        .build();
+        ).build();
 
-    let build = project().at("ws/builder")
+    let build = project()
+        .at("ws/builder")
         .file(
             "Cargo.toml",
             r#"
@@ -131,11 +127,11 @@ fn plugin_with_dynamic_native_dependency() {
             name = "builder"
             crate-type = ["dylib"]
         "#,
-        )
-        .file("src/lib.rs", "#[no_mangle] pub extern fn foo() {}")
+        ).file("src/lib.rs", "#[no_mangle] pub extern fn foo() {}")
         .build();
 
-    let foo = project().at("ws/foo")
+    let foo = project()
+        .at("ws/foo")
         .file(
             "Cargo.toml",
             r#"
@@ -147,8 +143,7 @@ fn plugin_with_dynamic_native_dependency() {
             [dependencies.bar]
             path = "bar"
         "#,
-        )
-        .file(
+        ).file(
             "src/main.rs",
             r#"
             #![feature(plugin)]
@@ -156,8 +151,7 @@ fn plugin_with_dynamic_native_dependency() {
 
             fn main() {}
         "#,
-        )
-        .file(
+        ).file(
             "bar/Cargo.toml",
             r#"
             [package]
@@ -170,8 +164,7 @@ fn plugin_with_dynamic_native_dependency() {
             name = "bar"
             plugin = true
         "#,
-        )
-        .file(
+        ).file(
             "bar/build.rs",
             r#"
             use std::path::PathBuf;
@@ -182,8 +175,7 @@ fn plugin_with_dynamic_native_dependency() {
                 println!("cargo:rustc-flags=-L {}/deps", src.parent().unwrap().display());
             }
         "#,
-        )
-        .file(
+        ).file(
             "bar/src/lib.rs",
             r#"
             #![feature(plugin_registrar, rustc_private)]
@@ -200,10 +192,9 @@ fn plugin_with_dynamic_native_dependency() {
                 unsafe { foo() }
             }
         "#,
-        )
-        .build();
+        ).build();
 
-    assert_that(build.cargo("build"), execs());
+    build.cargo("build").run();
 
     let src = workspace.root().join("target/debug");
     let lib = fs::read_dir(&src)
@@ -212,13 +203,9 @@ fn plugin_with_dynamic_native_dependency() {
         .find(|lib| {
             let lib = lib.file_name().unwrap().to_str().unwrap();
             lib.starts_with(env::consts::DLL_PREFIX) && lib.ends_with(env::consts::DLL_SUFFIX)
-        })
-        .unwrap();
+        }).unwrap();
 
-    assert_that(
-        foo.cargo("build").env("SRC", &lib).arg("-v"),
-        execs(),
-    );
+    foo.cargo("build -v").env("SRC", &lib).run();
 }
 
 #[test]
@@ -238,13 +225,12 @@ fn plugin_integration() {
             plugin = true
             doctest = false
         "#,
-        )
-        .file("build.rs", "fn main() {}")
+        ).file("build.rs", "fn main() {}")
         .file("src/lib.rs", "")
         .file("tests/it_works.rs", "")
         .build();
 
-    assert_that(p.cargo("test").arg("-v"), execs());
+    p.cargo("test -v").run();
 }
 
 #[test]
@@ -261,8 +247,7 @@ fn doctest_a_plugin() {
             [dependencies]
             bar = { path = "bar" }
         "#,
-        )
-        .file("src/lib.rs", "#[macro_use] extern crate bar;")
+        ).file("src/lib.rs", "#[macro_use] extern crate bar;")
         .file(
             "bar/Cargo.toml",
             r#"
@@ -275,11 +260,10 @@ fn doctest_a_plugin() {
             name = "bar"
             plugin = true
         "#,
-        )
-        .file("bar/src/lib.rs", "pub fn bar() {}")
+        ).file("bar/src/lib.rs", "pub fn bar() {}")
         .build();
 
-    assert_that(p.cargo("test").arg("-v"), execs());
+    p.cargo("test -v").run();
 }
 
 // See #1515
@@ -299,11 +283,11 @@ fn native_plugin_dependency_with_custom_ar_linker() {
             [lib]
             plugin = true
         "#,
-        )
-        .file("src/lib.rs", "")
+        ).file("src/lib.rs", "")
         .build();
 
-    let bar = project().at("bar")
+    let bar = project()
+        .at("bar")
         .file(
             "Cargo.toml",
             r#"
@@ -315,8 +299,7 @@ fn native_plugin_dependency_with_custom_ar_linker() {
             [dependencies.foo]
             path = "../foo"
         "#,
-        )
-        .file("src/lib.rs", "")
+        ).file("src/lib.rs", "")
         .file(
             ".cargo/config",
             &format!(
@@ -327,19 +310,17 @@ fn native_plugin_dependency_with_custom_ar_linker() {
         "#,
                 target
             ),
-        )
-        .build();
+        ).build();
 
-    assert_that(
-        bar.cargo("build").arg("--verbose"),
-        execs().with_status(101).with_stderr_contains(
+    bar.cargo("build --verbose")
+        .with_status(101)
+        .with_stderr_contains(
             "\
 [COMPILING] foo v0.0.1 ([..])
 [RUNNING] `rustc [..] -C ar=nonexistent-ar -C linker=nonexistent-linker [..]`
 [ERROR] [..]linker[..]
 ",
-        ),
-    );
+        ).run();
 }
 
 #[test]
@@ -363,8 +344,7 @@ fn panic_abort_plugins() {
             [dependencies]
             bar = { path = "bar" }
         "#,
-        )
-        .file("src/lib.rs", "")
+        ).file("src/lib.rs", "")
         .file(
             "bar/Cargo.toml",
             r#"
@@ -376,17 +356,15 @@ fn panic_abort_plugins() {
             [lib]
             plugin = true
         "#,
-        )
-        .file(
+        ).file(
             "bar/src/lib.rs",
             r#"
             #![feature(rustc_private)]
             extern crate syntax;
         "#,
-        )
-        .build();
+        ).build();
 
-    assert_that(p.cargo("build"), execs());
+    p.cargo("build").run();
 }
 
 #[test]
@@ -411,8 +389,7 @@ fn shared_panic_abort_plugins() {
             bar = { path = "bar" }
             baz = { path = "baz" }
         "#,
-        )
-        .file("src/lib.rs", "extern crate baz;")
+        ).file("src/lib.rs", "extern crate baz;")
         .file(
             "bar/Cargo.toml",
             r#"
@@ -427,18 +404,16 @@ fn shared_panic_abort_plugins() {
             [dependencies]
             baz = { path = "../baz" }
         "#,
-        )
-        .file(
+        ).file(
             "bar/src/lib.rs",
             r#"
             #![feature(rustc_private)]
             extern crate syntax;
             extern crate baz;
         "#,
-        )
-        .file("baz/Cargo.toml", &basic_manifest("baz", "0.0.1"))
+        ).file("baz/Cargo.toml", &basic_manifest("baz", "0.0.1"))
         .file("baz/src/lib.rs", "")
         .build();
 
-    assert_that(p.cargo("build"), execs());
+    p.cargo("build").run();
 }
