@@ -1,205 +1,143 @@
-use std::cmp::PartialEq;
-use std::collections::{BTreeMap, HashSet};
+use std::env;
 
-use cargo::core::dependency::Kind::{self, Development};
-use cargo::core::resolver::{self, Method};
-use cargo::core::source::{GitReference, SourceId};
-use cargo::core::{enable_nightly_features, Dependency, PackageId, Registry, Summary};
-use cargo::util::{CargoResult, Config, ToUrl};
+use cargo::core::dependency::Kind::Development;
+use cargo::core::{enable_nightly_features, Dependency};
+use cargo::util::Config;
 
 use support::project;
 use support::registry::Package;
+use support::resolver::{
+    assert_contains, assert_same, dep, dep_kind, dep_loc, dep_req, loc_names, names, pkg, pkg_dep,
+    pkg_id, pkg_loc, registry, registry_strategy, resolve, resolve_and_validated,
+    resolve_with_config, PrettyPrintRegistry, ToDep, ToPkgId,
+};
 
-fn resolve(
-    pkg: &PackageId,
-    deps: Vec<Dependency>,
-    registry: &[Summary],
-) -> CargoResult<Vec<PackageId>> {
-    resolve_with_config(pkg, deps, registry, None)
-}
+use proptest::collection::vec;
+use proptest::prelude::*;
 
-fn resolve_with_config(
-    pkg: &PackageId,
-    deps: Vec<Dependency>,
-    registry: &[Summary],
-    config: Option<&Config>,
-) -> CargoResult<Vec<PackageId>> {
-    struct MyRegistry<'a>(&'a [Summary]);
-    impl<'a> Registry for MyRegistry<'a> {
-        fn query(
-            &mut self,
-            dep: &Dependency,
-            f: &mut FnMut(Summary),
-            fuzzy: bool,
-        ) -> CargoResult<()> {
-            for summary in self.0.iter() {
-                if fuzzy || dep.matches(summary) {
-                    f(summary.clone());
-                }
-            }
-            Ok(())
+proptest! {
+    #![proptest_config(ProptestConfig {
+        // Note that this is a little low in terms of cases we'd like to test,
+        // but this number affects how long this function takes. It can be
+        // increased locally to execute more tests and try to find more bugs,
+        // but for now it's semi-low to run in a small-ish amount of time on CI
+        // and locally.
+        cases: 256,
+        max_shrink_iters:
+            if env::var("CI").is_ok() {
+                // This attempts to make sure that CI will fail fast,
+                0
+            } else {
+                // but that local builds will give a small clear test case.
+                ProptestConfig::default().max_shrink_iters
+            },
+        .. ProptestConfig::default()
+    })]
+    #[test]
+    fn passes_validation(
+        PrettyPrintRegistry(input) in registry_strategy(50, 20, 60)
+    )  {
+        let reg = registry(input.clone());
+        // there is only a small chance that eny one
+        // crate will be interesting.
+        // So we try some of the most complicated.
+        for this in input.iter().rev().take(20) {
+            let _ = resolve_and_validated(
+                &pkg_id("root"),
+                vec![dep_req(&this.name(), &format!("={}", this.version()))],
+                &reg,
+            );
         }
     }
-    let mut registry = MyRegistry(registry);
-    let summary = Summary::new(
-        pkg.clone(),
-        deps,
-        &BTreeMap::<String, Vec<String>>::new(),
-        None::<String>,
-        false,
-    ).unwrap();
-    let method = Method::Everything;
-    let resolve = resolver::resolve(
-        &[(summary, method)],
-        &[],
-        &mut registry,
-        &HashSet::new(),
-        config,
-        false,
-    )?;
-    let res = resolve.iter().cloned().collect();
-    Ok(res)
-}
+    #[test]
+    fn limited_independence_of_irrelevant_alternatives(
+        PrettyPrintRegistry(input) in registry_strategy(50, 20, 60),
+        indexs_to_unpublish in vec(any::<prop::sample::Index>(), 10)
+    )  {
+        let reg = registry(input.clone());
+        // there is only a small chance that eny one
+        // crate will be interesting.
+        // So we try some of the most complicated.
+        for this in input.iter().rev().take(10) {
+            let res = resolve(
+                &pkg_id("root"),
+                vec![dep_req(&this.name(), &format!("={}", this.version()))],
+                &reg,
+            );
 
-trait ToDep {
-    fn to_dep(self) -> Dependency;
-}
+            match res {
+                Ok(r) => {
+                    // If resolution was successful, then unpublishing a version of a crate
+                    // that was not selected should not change that.
+                    let not_selected: Vec<_> = input
+                        .iter()
+                        .cloned()
+                        .filter(|x| !r.contains(x.package_id()))
+                        .collect();
+                    if !not_selected.is_empty() {
+                        let indexs_to_unpublish: Vec<_> = indexs_to_unpublish.iter().map(|x| x.get(&not_selected)).collect();
 
-impl ToDep for &'static str {
-    fn to_dep(self) -> Dependency {
-        let url = "http://example.com".to_url().unwrap();
-        let source_id = SourceId::for_registry(&url).unwrap();
-        Dependency::parse_no_deprecated(self, Some("1.0.0"), &source_id).unwrap()
+                        let new_reg = registry(
+                            input
+                                .iter()
+                                .cloned()
+                                .filter(|x| !indexs_to_unpublish.contains(&x))
+                                .collect(),
+                        );
+
+                        let res = resolve(
+                            &pkg_id("root"),
+                            vec![dep_req(&this.name(), &format!("={}", this.version()))],
+                            &new_reg,
+                        );
+
+                        // Note: that we can not assert that the two `res` are identical
+                        // as the resolver does depend on irrelevant alternatives.
+                        // It uses how constrained a dependency requirement is
+                        // to determine what order to evaluate requirements.
+
+                        prop_assert!(
+                            res.is_ok(),
+                            "unpublishing {:?} stopped `{} = \"={}\"` from working",
+                            indexs_to_unpublish.iter().map(|x| x.package_id()).collect::<Vec<_>>(),
+                            this.name(),
+                            this.version()
+                        )
+                    }
+                }
+
+                Err(_) => {
+                    // If resolution was unsuccessful, then it should stay unsuccessful
+                    // even if any version of a crate is unpublished.
+                    let indexs_to_unpublish: Vec<_> = indexs_to_unpublish.iter().map(|x| x.get(&input)).collect();
+
+                    let new_reg = registry(
+                        input
+                            .iter()
+                            .cloned()
+                            .filter(|x| !indexs_to_unpublish.contains(&x))
+                            .collect(),
+                    );
+
+                    let res = resolve(
+                        &pkg_id("root"),
+                        vec![dep_req(&this.name(), &format!("={}", this.version()))],
+                        &new_reg,
+                    );
+
+                    prop_assert!(
+                        res.is_err(),
+                        "full index did not work for `{} = \"={}\"` but unpublishing {:?} fixed it!",
+                        this.name(),
+                        this.version(),
+                        indexs_to_unpublish.iter().map(|x| x.package_id()).collect::<Vec<_>>(),
+                    )
+                }
+            }
+        }
     }
 }
 
-impl ToDep for Dependency {
-    fn to_dep(self) -> Dependency {
-        self
-    }
-}
-
-trait ToPkgId {
-    fn to_pkgid(&self) -> PackageId;
-}
-
-impl<'a> ToPkgId for &'a str {
-    fn to_pkgid(&self) -> PackageId {
-        PackageId::new(*self, "1.0.0", &registry_loc()).unwrap()
-    }
-}
-
-impl<'a> ToPkgId for (&'a str, &'a str) {
-    fn to_pkgid(&self) -> PackageId {
-        let (name, vers) = *self;
-        PackageId::new(name, vers, &registry_loc()).unwrap()
-    }
-}
-
-impl<'a> ToPkgId for (&'a str, String) {
-    fn to_pkgid(&self) -> PackageId {
-        let (name, ref vers) = *self;
-        PackageId::new(name, vers, &registry_loc()).unwrap()
-    }
-}
-
-macro_rules! pkg {
-    ($pkgid:expr => [$($deps:expr),+]) => ({
-        let d: Vec<Dependency> = vec![$($deps.to_dep()),+];
-        let pkgid = $pkgid.to_pkgid();
-        let link = if pkgid.name().ends_with("-sys") {Some(pkgid.name().as_str())} else {None};
-
-        Summary::new(pkgid, d, &BTreeMap::<String, Vec<String>>::new(), link, false).unwrap()
-    });
-
-    ($pkgid:expr) => ({
-        let pkgid = $pkgid.to_pkgid();
-        let link = if pkgid.name().ends_with("-sys") {Some(pkgid.name().as_str())} else {None};
-        Summary::new(pkgid, Vec::new(), &BTreeMap::<String, Vec<String>>::new(), link, false).unwrap()
-    })
-}
-
-fn registry_loc() -> SourceId {
-    let remote = "http://example.com".to_url().unwrap();
-    SourceId::for_registry(&remote).unwrap()
-}
-
-fn pkg(name: &str) -> Summary {
-    let link = if name.ends_with("-sys") {
-        Some(name)
-    } else {
-        None
-    };
-    Summary::new(
-        pkg_id(name),
-        Vec::new(),
-        &BTreeMap::<String, Vec<String>>::new(),
-        link,
-        false,
-    ).unwrap()
-}
-
-fn pkg_id(name: &str) -> PackageId {
-    PackageId::new(name, "1.0.0", &registry_loc()).unwrap()
-}
-
-fn pkg_id_loc(name: &str, loc: &str) -> PackageId {
-    let remote = loc.to_url();
-    let master = GitReference::Branch("master".to_string());
-    let source_id = SourceId::for_git(&remote.unwrap(), master).unwrap();
-
-    PackageId::new(name, "1.0.0", &source_id).unwrap()
-}
-
-fn pkg_loc(name: &str, loc: &str) -> Summary {
-    let link = if name.ends_with("-sys") {
-        Some(name)
-    } else {
-        None
-    };
-    Summary::new(
-        pkg_id_loc(name, loc),
-        Vec::new(),
-        &BTreeMap::<String, Vec<String>>::new(),
-        link,
-        false,
-    ).unwrap()
-}
-
-fn dep(name: &str) -> Dependency {
-    dep_req(name, "1.0.0")
-}
-fn dep_req(name: &str, req: &str) -> Dependency {
-    let url = "http://example.com".to_url().unwrap();
-    let source_id = SourceId::for_registry(&url).unwrap();
-    Dependency::parse_no_deprecated(name, Some(req), &source_id).unwrap()
-}
-
-fn dep_loc(name: &str, location: &str) -> Dependency {
-    let url = location.to_url().unwrap();
-    let master = GitReference::Branch("master".to_string());
-    let source_id = SourceId::for_git(&url, master).unwrap();
-    Dependency::parse_no_deprecated(name, Some("1.0.0"), &source_id).unwrap()
-}
-fn dep_kind(name: &str, kind: Kind) -> Dependency {
-    dep(name).set_kind(kind).clone()
-}
-
-fn registry(pkgs: Vec<Summary>) -> Vec<Summary> {
-    pkgs
-}
-
-fn names<P: ToPkgId>(names: &[P]) -> Vec<PackageId> {
-    names.iter().map(|name| name.to_pkgid()).collect()
-}
-
-fn loc_names(names: &[(&'static str, &'static str)]) -> Vec<PackageId> {
-    names
-        .iter()
-        .map(|&(name, loc)| pkg_id_loc(name, loc))
-        .collect()
-}
-
 #[test]
 #[should_panic(expected = "assertion failed: !name.is_empty()")]
 fn test_dependency_with_empty_name() {
@@ -214,28 +152,16 @@ fn test_resolving_empty_dependency_list() {
     assert_eq!(res, names(&["root"]));
 }
 
-/// Assert `xs` contains `elems`
-fn assert_contains<A: PartialEq>(xs: &[A], elems: &[A]) {
-    for elem in elems {
-        assert!(xs.contains(elem));
-    }
-}
-
-fn assert_same<A: PartialEq>(a: &[A], b: &[A]) {
-    assert_eq!(a.len(), b.len());
-    assert_contains(b, a);
-}
-
 #[test]
 fn test_resolving_only_package() {
-    let reg = registry(vec![pkg("foo")]);
+    let reg = registry(vec![pkg!("foo")]);
     let res = resolve(&pkg_id("root"), vec![dep("foo")], &reg).unwrap();
     assert_same(&res, &names(&["root", "foo"]));
 }
 
 #[test]
 fn test_resolving_one_dep() {
-    let reg = registry(vec![pkg("foo"), pkg("bar")]);
+    let reg = registry(vec![pkg!("foo"), pkg!("bar")]);
     let res = resolve(&pkg_id("root"), vec![dep("foo")], &reg).unwrap();
     assert_same(&res, &names(&["root", "foo"]));
 }
@@ -252,7 +178,7 @@ fn test_resolving_transitive_deps() {
     let reg = registry(vec![pkg!("foo"), pkg!("bar" => ["foo"])]);
     let res = resolve(&pkg_id("root"), vec![dep("bar")], &reg).unwrap();
 
-    assert_contains(&res, &names(&["root", "foo", "bar"]));
+    assert_same(&res, &names(&["root", "foo", "bar"]));
 }
 
 #[test]
@@ -260,7 +186,7 @@ fn test_resolving_common_transitive_deps() {
     let reg = registry(vec![pkg!("foo" => ["bar"]), pkg!("bar")]);
     let res = resolve(&pkg_id("root"), vec![dep("foo"), dep("bar")], &reg).unwrap();
 
-    assert_contains(&res, &names(&["root", "foo", "bar"]));
+    assert_same(&res, &names(&["root", "foo", "bar"]));
 }
 
 #[test]
@@ -278,7 +204,8 @@ fn test_resolving_with_same_name() {
             dep_loc("bar", "http://second.example.com"),
         ],
         &reg,
-    ).unwrap();
+    )
+    .unwrap();
 
     let mut names = loc_names(&[
         ("foo", "http://first.example.com"),
@@ -302,9 +229,10 @@ fn test_resolving_with_dev_deps() {
         &pkg_id("root"),
         vec![dep("foo"), dep_kind("baz", Development)],
         &reg,
-    ).unwrap();
+    )
+    .unwrap();
 
-    assert_contains(&res, &names(&["root", "foo", "bar", "baz"]));
+    assert_same(&res, &names(&["root", "foo", "bar", "baz", "bat"]));
 }
 
 #[test]
@@ -313,7 +241,7 @@ fn resolving_with_many_versions() {
 
     let res = resolve(&pkg_id("root"), vec![dep("foo")], &reg).unwrap();
 
-    assert_contains(&res, &names(&[("root", "1.0.0"), ("foo", "1.0.2")]));
+    assert_same(&res, &names(&[("root", "1.0.0"), ("foo", "1.0.2")]));
 }
 
 #[test]
@@ -322,7 +250,7 @@ fn resolving_with_specific_version() {
 
     let res = resolve(&pkg_id("root"), vec![dep_req("foo", "=1.0.1")], &reg).unwrap();
 
-    assert_contains(&res, &names(&[("root", "1.0.0"), ("foo", "1.0.1")]));
+    assert_same(&res, &names(&[("root", "1.0.0"), ("foo", "1.0.1")]));
 }
 
 #[test]
@@ -339,7 +267,8 @@ fn test_resolving_maximum_version_with_transitive_deps() {
         &pkg_id("root"),
         vec![dep_req("foo", "1.0.0"), dep_req("bar", "1.0.0")],
         &reg,
-    ).unwrap();
+    )
+    .unwrap();
 
     assert_contains(
         &res,
@@ -379,14 +308,16 @@ fn test_resolving_minimum_version_with_transitive_deps() {
             false,
             &None,
             &["minimal-versions".to_string()],
-        ).unwrap();
+        )
+        .unwrap();
 
     let res = resolve_with_config(
         &pkg_id("root"),
         vec![dep_req("foo", "1.0.0"), dep_req("bar", "1.0.0")],
         &reg,
         Some(&config),
-    ).unwrap();
+    )
+    .unwrap();
 
     assert_contains(
         &res,
@@ -420,7 +351,8 @@ fn minimal_version_cli() {
             [dependencies]
             dep = "1.0"
         "#,
-        ).file("src/main.rs", "fn main() {}")
+        )
+        .file("src/main.rs", "fn main() {}")
         .build();
 
     p.cargo("generate-lockfile -Zminimal-versions")
@@ -445,7 +377,8 @@ fn resolving_incompat_versions() {
             &pkg_id("root"),
             vec![dep_req("foo", "=1.0.1"), dep("bar")],
             &reg
-        ).is_err()
+        )
+        .is_err()
     );
 }
 
@@ -524,7 +457,7 @@ fn resolving_allows_multiple_compatible_versions() {
 
     let res = resolve(&pkg_id("root"), vec![dep("bar")], &reg).unwrap();
 
-    assert_contains(
+    assert_same(
         &res,
         &names(&[
             ("root", "1.0.0"),
@@ -557,7 +490,7 @@ fn resolving_with_deep_backtracking() {
 
     let res = resolve(&pkg_id("root"), vec![dep_req("foo", "1")], &reg).unwrap();
 
-    assert_contains(
+    assert_same(
         &res,
         &names(&[
             ("root", "1.0.0"),
@@ -586,9 +519,10 @@ fn resolving_with_sys_crates() {
         &pkg_id("root"),
         vec![dep_req("d", "1"), dep_req("r", "1")],
         &reg,
-    ).unwrap();
+    )
+    .unwrap();
 
-    assert_contains(
+    assert_same(
         &res,
         &names(&[
             ("root", "1.0.0"),
@@ -697,7 +631,8 @@ fn resolving_with_many_equivalent_backtracking() {
         &pkg_id("root"),
         vec![dep_req("level0", "*"), dep_req("constrained", "*")],
         &reg,
-    ).unwrap();
+    )
+    .unwrap();
 
     assert_contains(
         &res,
@@ -714,7 +649,8 @@ fn resolving_with_many_equivalent_backtracking() {
         &pkg_id("root"),
         vec![dep_req("level0", "1.0.1"), dep_req("constrained", "*")],
         &reg,
-    ).unwrap();
+    )
+    .unwrap();
 
     assert_contains(
         &res,
@@ -864,7 +800,8 @@ fn resolving_with_constrained_cousins_backtrack() {
         &pkg_id("root"),
         vec![dep_req("level0", "*"), dep_req("constrained", "2.0.0")],
         &reg,
-    ).unwrap();
+    )
+    .unwrap();
 
     assert_contains(
         &res,
@@ -952,9 +889,10 @@ fn resolving_with_constrained_sibling_transitive_dep_effects() {
 
     let res = resolve(&pkg_id("root"), vec![dep_req("A", "1")], &reg).unwrap();
 
-    assert_contains(
+    assert_same(
         &res,
         &names(&[
+            ("root", "1.0.0"),
             ("A", "1.0.0"),
             ("B", "1.0.0"),
             ("C", "1.0.0"),
@@ -963,6 +901,174 @@ fn resolving_with_constrained_sibling_transitive_dep_effects() {
     );
 }
 
+#[test]
+fn incomplete_information_skiping() {
+    // When backtracking due to a failed dependency, if Cargo is
+    // trying to be clever and skip irrelevant dependencies, care must
+    // be taken to not miss the transitive effects of alternatives.
+    // Fuzzing discovered that for some reason cargo was skiping based
+    // on incomplete information in the following case:
+    // minimized bug found in:
+    // https://github.com/rust-lang/cargo/commit/003c29b0c71e5ea28fbe8e72c148c755c9f3f8d9
+    let input = vec![
+        pkg!(("a", "1.0.0")),
+        pkg!(("a", "1.1.0")),
+        pkg!("b" => [dep("a")]),
+        pkg!(("c", "1.0.0")),
+        pkg!(("c", "1.1.0")),
+        pkg!("d" => [dep_req("c", "=1.0")]),
+        pkg!(("e", "1.0.0")),
+        pkg!(("e", "1.1.0") => [dep_req("c", "1.1")]),
+        pkg!("to_yank"),
+        pkg!(("f", "1.0.0") => [
+            dep("to_yank"),
+            dep("d"),
+        ]),
+        pkg!(("f", "1.1.0") => [dep("d")]),
+        pkg!("g" => [
+            dep("b"),
+            dep("e"),
+            dep("f"),
+        ]),
+    ];
+    let reg = registry(input.clone());
+
+    let res = resolve(&pkg_id("root"), vec![dep("g")], &reg).unwrap();
+    let package_to_yank = "to_yank".to_pkgid();
+    // this package is not used in the resolution.
+    assert!(!res.contains(&package_to_yank));
+    // so when we yank it
+    let new_reg = registry(
+        input
+            .iter()
+            .cloned()
+            .filter(|x| &package_to_yank != x.package_id())
+            .collect(),
+    );
+    assert_eq!(input.len(), new_reg.len() + 1);
+    // it should still build
+    assert!(resolve(&pkg_id("root"), vec![dep("g")], &new_reg).is_ok());
+}
+
+#[test]
+fn incomplete_information_skiping_2() {
+    // When backtracking due to a failed dependency, if Cargo is
+    // trying to be clever and skip irrelevant dependencies, care must
+    // be taken to not miss the transitive effects of alternatives.
+    // Fuzzing discovered that for some reason cargo was skiping based
+    // on incomplete information in the following case:
+    // https://github.com/rust-lang/cargo/commit/003c29b0c71e5ea28fbe8e72c148c755c9f3f8d9
+    let input = vec![
+        pkg!(("b", "3.8.10")),
+        pkg!(("b", "8.7.4")),
+        pkg!(("b", "9.4.6")),
+        pkg!(("c", "1.8.8")),
+        pkg!(("c", "10.2.5")),
+        pkg!(("d", "4.1.2") => [
+            dep_req("bad", "=6.10.9"),
+        ]),
+        pkg!(("d", "5.5.6")),
+        pkg!(("d", "5.6.10")),
+        pkg!(("to_yank", "8.0.1")),
+        pkg!(("to_yank", "8.8.1")),
+        pkg!(("e", "4.7.8") => [
+            dep_req("d", ">=5.5.6, <=5.6.10"),
+            dep_req("to_yank", "=8.0.1"),
+        ]),
+        pkg!(("e", "7.4.9") => [
+            dep_req("bad", "=4.7.5"),
+        ]),
+        pkg!("f" => [
+            dep_req("d", ">=4.1.2, <=5.5.6"),
+        ]),
+        pkg!("g" => [
+            dep("bad"),
+        ]),
+        pkg!(("h", "3.8.3") => [
+            dep_req("g", "*"),
+        ]),
+        pkg!(("h", "6.8.3") => [
+            dep("f"),
+        ]),
+        pkg!(("h", "8.1.9") => [
+            dep_req("to_yank", "=8.8.1"),
+        ]),
+        pkg!("i" => [
+            dep_req("b", "*"),
+            dep_req("c", "*"),
+            dep_req("e", "*"),
+            dep_req("h", "*"),
+        ]),
+    ];
+    let reg = registry(input.clone());
+
+    let res = resolve(&pkg_id("root"), vec![dep("i")], &reg).unwrap();
+    let package_to_yank = ("to_yank", "8.8.1").to_pkgid();
+    // this package is not used in the resolution.
+    assert!(!res.contains(&package_to_yank));
+    // so when we yank it
+    let new_reg = registry(
+        input
+            .iter()
+            .cloned()
+            .filter(|x| &package_to_yank != x.package_id())
+            .collect(),
+    );
+    assert_eq!(input.len(), new_reg.len() + 1);
+    // it should still build
+    assert!(resolve(&pkg_id("root"), vec![dep("i")], &new_reg).is_ok());
+}
+
+#[test]
+fn incomplete_information_skiping_3() {
+    // When backtracking due to a failed dependency, if Cargo is
+    // trying to be clever and skip irrelevant dependencies, care must
+    // be taken to not miss the transitive effects of alternatives.
+    // Fuzzing discovered that for some reason cargo was skiping based
+    // on incomplete information in the following case:
+    // minimized bug found in:
+    // https://github.com/rust-lang/cargo/commit/003c29b0c71e5ea28fbe8e72c148c755c9f3f8d9
+    let input = vec![
+        pkg!{("to_yank", "3.0.3")},
+        pkg!{("to_yank", "3.3.0")},
+        pkg!{("to_yank", "3.3.1")},
+        pkg!{("a", "3.3.0") => [
+            dep_req("to_yank", "=3.0.3"),
+        ] },
+        pkg!{("a", "3.3.2") => [
+            dep_req("to_yank", "<=3.3.0"),
+        ] },
+        pkg!{("b", "0.1.3") => [
+            dep_req("a", "=3.3.0"),
+        ] },
+        pkg!{("b", "2.0.2") => [
+            dep_req("to_yank", "3.3.0"),
+            dep_req("a", "*"),
+        ] },
+        pkg!{("b", "2.3.3") => [
+            dep_req("to_yank", "3.3.0"),
+            dep_req("a", "=3.3.0"),
+        ] },
+    ];
+    let reg = registry(input.clone());
+
+    let res = resolve(&pkg_id("root"), vec![dep_req("b", "*")], &reg).unwrap();
+    let package_to_yank = ("to_yank", "3.0.3").to_pkgid();
+    // this package is not used in the resolution.
+    assert!(!res.contains(&package_to_yank));
+    // so when we yank it
+    let new_reg = registry(
+        input
+            .iter()
+            .cloned()
+            .filter(|x| &package_to_yank != x.package_id())
+            .collect(),
+    );
+    assert_eq!(input.len(), new_reg.len() + 1);
+    // it should still build
+    assert!(resolve(&pkg_id("root"), vec![dep_req("b", "*")], &new_reg).is_ok());
+}
+
 #[test]
 fn resolving_but_no_exists() {
     let reg = registry(vec![]);
@@ -999,9 +1105,10 @@ fn hard_equality() {
         &pkg_id("root"),
         vec![dep_req("bar", "1"), dep_req("foo", "=1.0.0")],
         &reg,
-    ).unwrap();
+    )
+    .unwrap();
 
-    assert_contains(
+    assert_same(
         &res,
         &names(&[("root", "1.0.0"), ("foo", "1.0.0"), ("bar", "1.0.0")]),
     );
