@@ -557,7 +557,8 @@ fn share_dependencies() {
         .with_stderr(
             "\
 [UPDATING] `[..]` index
-[DOWNLOADING] dep1 v0.1.3 ([..])
+[DOWNLOADING] crates ...
+[DOWNLOADED] dep1 v0.1.3 ([..])
 [COMPILING] dep1 v0.1.3
 [COMPILING] foo v0.1.0 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
@@ -600,7 +601,8 @@ fn fetch_fetches_all() {
         .with_stderr(
             "\
 [UPDATING] `[..]` index
-[DOWNLOADING] dep1 v0.1.3 ([..])
+[DOWNLOADING] crates ...
+[DOWNLOADED] dep1 v0.1.3 ([..])
 ",
         ).run();
 }
@@ -650,7 +652,8 @@ fn lock_works_for_everyone() {
     p.cargo("build")
         .with_stderr(
             "\
-[DOWNLOADING] dep2 v0.1.0 ([..])
+[DOWNLOADING] crates ...
+[DOWNLOADED] dep2 v0.1.0 ([..])
 [COMPILING] dep2 v0.1.0
 [COMPILING] foo v0.1.0 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
@@ -661,7 +664,8 @@ fn lock_works_for_everyone() {
         .cwd(p.root().join("bar"))
         .with_stderr(
             "\
-[DOWNLOADING] dep1 v0.1.0 ([..])
+[DOWNLOADING] crates ...
+[DOWNLOADED] dep1 v0.1.0 ([..])
 [COMPILING] dep1 v0.1.0
 [COMPILING] bar v0.1.0 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
@@ -916,7 +920,7 @@ workspace: [..]
 
 this may be fixable by ensuring that this crate is depended on by the workspace \
 root: [..]
-[CREATED] library `bar` project
+[CREATED] library `bar` package
 ",
         ).run();
 }
